package Formatter::Base;

#=============================================================================
# Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
# All rights reserved.
#
# This file is part of Relax. 
#
# Relax is distributed with NO WARRANTY OF ANY KIND. No author or distributor
# accepts any responsibility for the consequences of using it, or for whether
# it serves any particular purpose or works at all, unless he or she says so
# in writing. Refer to the GNU General Public License (the "License") for
# full details. 
#
# Module name   : Formatter::Base.pm
# Version       : 1.00
# Purpose       : Formatter base class.
# Date          : 2003-01-06
# Updates       : 1.00 First release. (2003-01-06)
# Notes         : -
#
#=============================================================================

use strict;

BEGIN {
    use Exporter   ();
    use vars       qw ($VERSION @ISA);
    $VERSION     = do {
        my @r = q$Revision: 1.00 $=~/\d+/g;sprintf '%d.'.'%02d'x$#r,@r
    };
    @ISA         = qw (Exporter);
};




sub new {
    my ($proto) = shift;
    my ($class) = ref ($proto) || $proto;
    my ($self)  = {};

    $self -> {"INITIALIZED"}   = 0;
    $self -> {"TITLE"}         = undef;
    $self -> {"LANGUAGE"}      = undef;
    $self -> {"SPLIT_REPORT"}  = undef;
    $self -> {"REPORT_ID"}     = undef;
    $self -> {"REPORT_TITLE"}  = undef;
    $self -> {"SUMMARY"}       = undef;
    $self -> {"STREAMABLE"}    = 1;
    $self -> {"CONTENT_STR"}   = "";
    $self -> {"REPORT_HASH"}   = {};
    $self -> {"ROW_NUMBER"}    = 0;
    $self -> {"COLUMN_NUMBER"} = 0;

    bless ($self, $class);

    return $self;
};


sub init_formatter {
    my ($self)  = shift;
    my (%param) = @_;

    $self -> initialized (1);
};


sub initialized {
    my ($self) = shift;

    if (@_) {
        $self -> {"INITIALIZED"} = shift;
    };

    return $self -> {"INITIALIZED"};
};


sub streamable_format {
    my ($self) = shift;

    if (@_) {
        $self -> {"STREAMABLE"} = shift;
    };

    return $self -> {"STREAMABLE"};
};


sub split_report {
    my ($self) = shift;

    if (@_) {
        $self -> {"SPLIT_REPORT"} = shift;
    };

    return $self -> {"SPLIT_REPORT"};
};


sub summary {
    my ($self) = shift;

    if (@_) {
        $self -> {"SUMMARY"} = shift;
    };

    return $self -> {"SUMMARY"};
};


sub row_number {
    my ($self) = shift;

    if (@_) {
        $self -> {"ROW_NUMBER"} = shift;
    };

    return $self -> {"ROW_NUMBER"};
};


sub column_number {
    my ($self) = shift;

    if (@_) {
        $self -> {"COLUMN_NUMBER"} = shift;
    };

    return $self -> {"COLUMN_NUMBER"};
};


sub content_str {
    my ($self) = shift;

    if (@_) {
        $self -> {"CONTENT_STR"} = shift;
    };

    return $self -> {"CONTENT_STR"};
};


sub add_to_content_str {
    my ($self)        = shift;
    my ($new_content) = @_;

    $self -> {"CONTENT_STR"} .= $new_content;

    return $new_content;
};


sub ZZZadd_to_content_str {
    my ($self)        = shift;
    my ($new_content) = shift;

    my ($old_content) = $self -> content_str ();

    $self -> content_str ($old_content . $new_content);

    return $new_content;
};


sub get_report {
    my ($self)  = shift;
    my ($key)   = shift;

    return $self -> {"REPORT_HASH"} -> {$key};
};


#sub get_report_content {
#    my ($self)  = shift;
#    my ($key)   = shift;
#
#    return $self -> {"REPORT_HASH"} -> {$key} -> {"report"};
#};


#sub get_report_title {
#    my ($self)  = shift;
#    my ($key)   = shift;
#
#    return $self -> {"REPORT_HASH"} -> {$key} -> {"title"};
#};


sub add_report {
    my ($self)  = shift;

    $self -> {"REPORT_HASH"} -> {$self -> id ()} = {
        "report"        => $self -> content_str (),
        "report_title"  => $self -> report_title ()
    };
};


sub ZZZadd_report {
    my ($self)  = shift;

    print $self -> title (), "+++++++\n";
    $self -> {"REPORT_HASH"} -> {$self -> id ()} = {
        "report" => $self -> content_str (),
        "title"  => $self -> title ()
    };
};



sub language {
    my ($self) = shift;
    my ($lang) = shift;

    if ($lang) {
        $lang =~ s/\_/\-/;
        $self -> {"LANGUAGE"} = $lang;
    };

    return $self -> {"LANGUAGE"};
};


sub title {
    my ($self) = shift;

    if (@_) {
        $self -> {"TITLE"} = shift;
    };

    return $self -> {"TITLE"};
};


sub id {
    my ($self) = shift;

    if (@_) {
        $self -> {"REPORT_ID"} = shift;
    };

    return $self -> {"REPORT_ID"};
};


sub report_title {
    my ($self) = shift;

    if (@_) {
        $self -> {"REPORT_TITLE"} = shift;
    };

    return $self -> {"REPORT_TITLE"};
};


sub begin_document {
    my ($self)  = shift;
    my (%param) = @_;

    $self -> title ($param {-title});
    $self -> language ($param {-language});
    $self -> split_report ($param {-split});
};


sub begin_report {
    my ($self)  = shift;
    my (%param) = @_;

    $self -> id ($param {-id});
    $self -> report_title ($param {-report_title}); # caption
    $self -> summary ($param {-summary});

#    $self -> title ($param {-title});
#    $self -> language ($param {-language});
#    $self -> id ($param {-id});
#    $self -> split_report ($param {-split});
#    $self -> summary ($param {-summary});
};


sub end_report {
    my ($self)  = shift;
    my (%param) = @_;

    $self -> add_report (
        $self -> id (),
        $self -> content_str ()
    );

    $self -> content_str ("");
};


sub begin_division {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_division {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_table {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_table {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_caption {
    my ($self)  = shift;
    my (%param) = @_;
};

sub caption {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_caption {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_table_header {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_table_header {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_table_body {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_table_body {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_row {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_row {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_header_row {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_header_row {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_item {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_item {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_header_item {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_header_item {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_list {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_list {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_list_item {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_list_item {
    my ($self)  = shift;
    my (%param) = @_;
};


sub item {
    my ($self)  = shift;
    my (%param) = @_;
};


sub header_item {
    my ($self)  = shift;
    my (%param) = @_;
};


sub url_item {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_heading {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_heading {
    my ($self)  = shift;
    my (%param) = @_;
};


sub heading_item {
    my ($self)  = shift;
    my (%param) = @_;
};


sub begin_paragraph {
    my ($self)  = shift;
    my (%param) = @_;
};


sub end_paragraph {
    my ($self)  = shift;
    my (%param) = @_;
};


sub text_item {
    my ($self)  = shift;
    my (%param) = @_;
};


sub escape_meta_data {
    my ($self)  = shift;
    my ($str) = @_;

    return $str;
};


sub template {
    my ($self)  = shift;
    my (%param) = @_;

    return ""; #undef
};




1;
