package Formatter::CSV;

#=============================================================================
# Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
# All rights reserved.
#
# This file is part of Relax. 
#
# Relax is distributed with NO WARRANTY OF ANY KIND. No author or distributor
# accepts any responsibility for the consequences of using it, or for whether
# it serves any particular purpose or works at all, unless he or she says so
# in writing. Refer to the GNU General Public License (the "License") for
# full details. 
#
# Module name   : Formatter::CSV.pm
# Version       : 1.00
# Purpose       : Formats reports in CSV (Comma separated values) format.
# Date          : 2003-01-27
# Updates       : 1.00 First release. (2003-01-27)
# Notes         : http://www.wotsit.org/download.asp?f=csv
#                 http://www.opengroup.org/onlinepubs/7908799/xbd/locale.html
#
#=============================================================================

use strict;

BEGIN {
    use Exporter   ();
    use vars       qw ($VERSION @ISA);
    $VERSION     = do {
        my @r = q$Revision: 1.01 $=~/\d+/g;sprintf '%d.'.'%02d'x$#r,@r
    };
    @ISA         = qw (Exporter Formatter::Text);
};




sub new {
    my ($proto) = shift;
    my ($class) = ref ($proto) || $proto;
    my $self    = $class -> SUPER::new (@_);
    my (%params) = @_;

    $self -> {"DATA_SEPARATOR"} = $params {-data_separator} || ",";
    $self -> {"DECIMEL_POINT"} = $params {-decimal_point} || ".";

    bless ($self, $class);

    return $self;
};


sub caption {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($param {-value});
};


sub end_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($self -> {"DATA_SEPARATOR"});
};


sub end_header_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($self -> {"DATA_SEPARATOR"});
};


sub escape {
    my ($self)  = shift;
    my ($value) = @_;

    my ($data_separator) = $self -> {"DATA_SEPARATOR"};
    my ($decimal_point) = $self -> {"DECIMEL_POINT"};

    $value = $value || "";
    $value =~ s|"|""|g;
    # Excell doesn't seem to like if the first character of the value
    # is "+" or "-". It doesn't seem to correctly handle "'" to escape
    # this situation. Therefore I'm using " " to take care of this.
    $value =~ s|^([\+\-].+)| $1|g;

    if ($value =~ /\d\.\d\d\%$/) {
        $value =~ s/\./$decimal_point/;
    }
    elsif ($value =~ /\"\"/) {
        $value = '"' . $value . '"';
    }
    elsif ($value =~ /$data_separator/) {
        $value = '"' . $value . '"';
    };

    return $value;
};


sub ZZZescape {
    my ($self)  = shift;
    my ($value) = @_;

    my ($data_separator) = $self -> {"DATA_SEPARATOR"};
    my ($decimal_point) = $self -> {"DECIMEL_POINT"};

    $value = $value || "";
    $value =~ s|"|""|g;

    if ($value =~ /\d\.\d\d\%$/) {
        $value =~ s/\./$decimal_point/;
    }
    elsif ($value =~ /\"\"/) {
        $value = '"' . $value . '"';
    }
    elsif ($value =~ /$data_separator/) {
        $value = '"' . $value . '"';
    };

    return $value;
};


sub item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($self -> escape ($param {-value}));
};


sub text_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($self -> escape ($param {-value}));
};


sub url_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($self -> escape ($param {-url_text}));
};


sub heading_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($param {-value} || "");
};


1;

