package Formatter::Html;

#=============================================================================
# Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
# All rights reserved.
#
# This file is part of Relax.
#
# Relax is distributed with NO WARRANTY OF ANY KIND. No author or distributor
# accepts any responsibility for the consequences of using it, or for whether
# it serves any particular purpose or works at all, unless he or she says so
# in writing. Refer to the GNU General Public License (the "License") for
# full details.
#
# Module name   : Formatter::Html.pm
# Version       : 1.01
# Purpose       : Formats reports to HTML format.
# Date          : 2003-07-20
# Updates       : 1.00 First release. (2002-12-17)
#                 1.01 Removed unnecessary quotation marks from  (2003-07-20)
#                        meta charset.
# Notes         : -
#
#=============================================================================

use strict;

BEGIN {
    use Exporter   ();
    use vars       qw ($VERSION @ISA);
    $VERSION     = do {
        my @r = q$Revision: 1.00 $=~/\d+/g;sprintf '%d.'.'%02d'x$#r,@r
    };
    @ISA         = qw (Exporter Formatter::Xhtml);
};




sub begin_division {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str (
        '<div' .
        ($param {-id} ? ' id="' . $param {-id} . '"' : '') .
        ($param {-class} ? ' class="' . $param {-class} . '"' : '') .
        '>' .
        "\n" .
        ($param {-id} ?
            '<a ' .
            'name="' .  $param {-id} . '"' .
            '> </a>' .
            "\n"
            :
            ''
        )
    );
};


sub template {
    my ($self)  = shift;
    my (%param) = @_;

    my ($stylesheet) = $self -> css_stylesheet ();
    my ($print_stylesheet) = $self -> css_print_stylesheet ();

    return <<END
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN"
    "http://www.w3.org/TR/REC-html40/loose.dtd">
<HTML LANG="<!--[report_language]-->">
  <HEAD>
    <TITLE><!--[report_title]--></TITLE>
    <LINK HREF="mailto:ktmatu\@surfeu.fi.remove.invalid" REV="made">
    <META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=<!--[charset]-->">
    <META NAME="description" CONTENT="Relax Referrer Log Analysis Report">
    <META NAME="keywords" CONTENT="Relax">
    <META NAME="robots" CONTENT="noindex,nofollow">
    <META NAME="DC.Date.Created" CONTENT="<!--[date]-->">
    <META NAME="DC.Creator" CONTENT="<!--[relax_version]-->">
    <META NAME="useless" CONTENT="meta tag">
    <STYLE TYPE="text/css" MEDIA="screen">
    <!--
$stylesheet
    -->
    </STYLE>
    <STYLE TYPE="text/css" MEDIA="print">
    <!--
$print_stylesheet
    -->
    </STYLE>
  </HEAD>
  <BODY>
<!--[report_header]-->
<!--[report_content]-->
<!--[report_navigation]-->
<!--[report_footer]-->
  </BODY>
</HTML>
END
};




1;

