package Formatter::Text;

#=============================================================================
# Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
# All rights reserved.
#
# This file is part of Relax. 
#
# Relax is distributed with NO WARRANTY OF ANY KIND. No author or distributor
# accepts any responsibility for the consequences of using it, or for whether
# it serves any particular purpose or works at all, unless he or she says so
# in writing. Refer to the GNU General Public License (the "License") for
# full details. 
#
# Module name   : Formatter::Text.pm
# Version       : 1.01
# Purpose       : Formats reports in plain text.
# Date          : 2003-01-15
# Updates       : 1.00 First release. (2003-01-05)
#                 1.01 end_heading method added. (2003-01-15)
# Notes         : -
#
#=============================================================================

use strict;

BEGIN {
    use Exporter   ();
    use vars       qw ($VERSION @ISA);
    $VERSION     = do {
        my @r = q$Revision: 1.01 $=~/\d+/g;sprintf '%d.'.'%02d'x$#r,@r
    };
    @ISA         = qw (Exporter Formatter::Base);
};




sub new {
    my ($proto) = shift;
    my ($class) = ref ($proto) || $proto;
    my $self    = $class -> SUPER::new (@_);

    bless ($self, $class);

    return $self;
};


sub begin_division {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("");
};


sub end_division {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("\n\n")
};


sub end_table {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("\n");
};


sub caption {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str (uc ($param {-value}));
};


sub end_caption {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("\n\n");
};


sub end_row {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("\n");
};


sub begin_header_row {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> begin_row (%param);
};


sub end_header_row {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> end_row (%param);
};


sub end_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str (" ");
};


sub end_header_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str (" ");
};


sub do_item {
    my ($self)  = shift;
    my (%param) = @_;

    my ($field) = "";
    my ($value) = $param {-value} || "";
    my ($align) = lc ($param {-align}) || "left";
    my ($length)= $param {-length} || 1;
    my ($left)  = undef;
    my ($right) = undef;

    if ((length ($value) > $length) && ($value !~ /\D/)) {
        $field = "#" x $length;
    }
    elsif ((length ($value) > $length) && ($value =~ /\D/)) {
        $field = "#" x $length;
        if ($length > 6) {
            $left  = int (($length - 3) / 2); # 3 = length "..."
            $right = $length - $left - 3;
            $field = substr ($value, 0, $left) .
                     "..." .
                     substr ($value, -$right);
        };
    }
    else {
        $field = $self -> align ($value, $align, $length);
    };

    return $field;
};


sub item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($self -> do_item (%param));
};


sub header_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> item (%param);
};


sub begin_list {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("\n");
};


sub end_list {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("\n");
};


sub begin_list_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str (" * ");
};


sub end_list_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("\n");
};


sub heading_item {
    my ($self)  = shift;
    my (%param) = @_;

    my ($level) = $param {-level} || "1";
    my ($underline) = "";

    if ($level eq "1") {
        $underline = "\n" . "=" x length ($param {-value});
    };

    return $self -> add_to_content_str (uc ($param {-value}) . $underline);
};


sub end_heading {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("\n\n");
};


sub begin_paragraph {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("\n");
};


sub end_paragraph {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ("\n\n");
};


sub text_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($param {-value});
};


sub url_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($param {-url_text});
};


sub align {
    my ($self)  = shift;

    my ($value) = shift;
    my ($align) = shift;
    my ($length)= shift;

    my ($field) = "";

    $field = " " x $length;
    if ($align eq "left") {
        $field = $value . $field;
        $field = substr ($field, 0, $length);
    }
    elsif ($align eq "center") {
        $field = " " x int ( ($length - length ($value)) / 2) .
            $value . $field;
        $field = substr ($field, 0, $length);
    }
    else {
        $field .= $value;
        $field = substr ($field, -$length);
    };

    return $field;
};


sub template {
    my ($self)  = shift;
    my (%param) = @_;

    return <<END
<!--[report_header]-->
<!--[report_content]-->
<!--[report_footer]-->
END
};


1;

