package Formatter::Textless;

#=============================================================================
# Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
# All rights reserved.
#
# This file is part of Relax. 
#
# Relax is distributed with NO WARRANTY OF ANY KIND. No author or distributor
# accepts any responsibility for the consequences of using it, or for whether
# it serves any particular purpose or works at all, unless he or she says so
# in writing. Refer to the GNU General Public License (the "License") for
# full details. 
#
# Module name   : Formatter::Textless.pm
# Version       : 1.00
# Purpose       : Formats reports suitable for Less pager program.
# Date          : 2002-12-17
# Updates       : 1.00 First release. (2002-12-17)
# Notes         : Reports created with this formatter are easier to read
#                   than plain text reports because Less understands nroff
#                   bold (character, backspace, character) and underline
#                   (character, backspace, underscore) formattings.
#                 Less home page: http://www.greenwoodsoftware.com/less/
#
#=============================================================================

use strict;

BEGIN {
    use Exporter   ();
    use vars       qw ($VERSION @ISA);
    $VERSION     = do {
        my @r = q$Revision: 1.00 $=~/\d+/g;sprintf '%d.'.'%02d'x$#r,@r
    };
    @ISA         = qw (Exporter Formatter::Text);
};




sub underline {
    my ($self) = shift;
    my ($str)  = shift;

    my ($underline_str) = "";
    my ($ch) = "";

    foreach $ch (split(//, $str)) {
        $underline_str .= "_" . chr (8) . $ch; # ^H
    };

    return $underline_str;
};


sub bold {
    my ($self) = shift;
    my ($str)  = shift;

    my ($bold_str) = "";
    my ($ch) = "";

    foreach $ch (split(//, $str)) {
        $bold_str .= $ch . chr (8) . $ch; # ^H
    };

    return $bold_str;
};


sub caption {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($self -> bold (uc ($param {-value})));
};


sub begin_list_item {
    my ($self)  = shift;
    my (%param) = @_;

    return $self -> add_to_content_str ($self -> bold (" * "));
};


sub heading_item {
    my ($self)  = shift;
    my (%param) = @_;

    my ($level) = $param {-level} || "1";
    my ($underline) = "";

    if ($level eq "1") {
        $underline = "\n" . $self -> bold ("=" x length ($param {-value}));
    };

    return $self -> add_to_content_str (
        $self -> bold (uc ($param {-value})) . $underline
    );
};


sub item {
    my ($self)  = shift;
    my (%param) = @_;

    my ($field) = "";
    my ($value) = $param {-value} || "";
    my ($align) = lc ($param {-align}) || "left";
    my ($length)= $param {-length} || 1;
    my ($class_str) = $param {-class} || "";

    if ($class_str eq "rsh") {
        # report-subhead
        if ((length ($value) * 3) > ($length * 3)) {
            # *3 = x^Hx
            $field = $self -> do_item (%param);
        }
        else {
            $field = $self -> align ($value, $align, $length);
        };

        $field = $self -> bold ($field)
    }
    else {
        $field = $self -> do_item (%param);
    };

    return $self -> add_to_content_str ($field);
};


sub header_item {
    my ($self)  = shift;
    my (%param) = @_;

    my ($field) = "";
    my ($value) = $param {-value} || "";
    my ($align) = lc ($param {-align}) || "left";
    my ($length)= $param {-length} || 1;

    if ((length ($value) * 3) > ($length * 3)) {
        $field = "#" x $length;
    }
    else {
        $field = $self -> align ($value, $align, $length);
    };

    return $self -> add_to_content_str ($self -> underline ($field));
};


sub url_item {
    my ($self)  = shift;
    my (%param) = @_;

    my ($field) = "";
    my ($class_str) = $param {-class} || "";

    # field length is not checked at all!
    $field = $param {-url_text};
    if ($class_str eq "rsh") {
        # report-subhead
        $field = $self -> bold ($param {-url_text})
    };

    return $self -> add_to_content_str ($field);
};


1;
