package Messages;

#=============================================================================
# Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
# All rights reserved.
#
# This file is part of Relax. 
#
# Relax is distributed with NO WARRANTY OF ANY KIND. No author or distributor
# accepts any responsibility for the consequences of using it, or for whether
# it serves any particular purpose or works at all, unless he or she says so
# in writing. Refer to the GNU General Public License (the "License") for
# full details. 
#
# Module name   : Messages.pm
# Version       : 1.19
# Purpose       : Translated messages module.
# Date          : 2003-08-13
# Updates       : 1.00 First release. (2002-11-28)
#                 1.10 A lot of new English and Finnish  (2003-01-14)
#                        entries.
#                 1.11 Added no_item_numbers related messages. (2003-01-15)
#                 1.12 Added run_time related messages. (2003-01-23)
#                 1.13 Added decimal_point and data_separator  (2003-01-27)
#                        messages.
#                 1.14 Added url_word_break related messages. (2003-01-28)
#                 1.15 Deleted -html, added report_format  (2003-02-04)
#                        messages, fixed typos.
#                 1.16 French messages updated by Rodolphe  (2003-03-16)
#                        Sibert.
#                 1.17 Date related messages added.  (2003-07-17)
#                 1.18 -rel report related messages added.  (2003-07-28)
#                 1.19 French messages updated by Rodolphe  (2003-08-13)
#                        Sibert.
#
# Notes         : See the "init" part of this module if you are interested
#                   in providing new translation.
#
#=============================================================================

use strict;

BEGIN {
    use Exporter   ();
    use vars       qw ($VERSION @ISA);
    $VERSION     = do {
        my @r = q$Revision: 1.19 $=~/\d+/g;sprintf '%d.'.'%02d'x$#r,@r
    };
    @ISA         = qw (Exporter);
};




sub new {
    my ($proto) = shift;
    my (%param) = @_;
    my ($class) = ref ($proto) || $proto;
    my ($self)  = {};

    bless ($self, $class);

    $self -> {"MESSAGES"}    = $self -> init ();
    $self -> {"LANGUAGE"}    = $param {-language};
    # $self -> {"LANGUAGE_OK"} = $self -> {"MESSAGES"} -> $param {-language};

    return $self;
};




#-----------------------------------------------------------------------------
# Function get_msg
#
# Purpose  : Returns translated message.
#
# Input    : $lang, String, Language indicator.
#            $msg_name, String, Message "constant".
# Output   : string
# Error
# handling : English translation with " [Translation missing!] " is returned
#              if the translation doesn't exist.
#
#-----------------------------------------------------------------------------
sub get_msg {
    my ($self)     = shift;
    my ($lang)     = shift;
    my ($msg_name) = shift;

    my ($msg) = "";

    if ($self -> {"MESSAGES"} -> {$lang}{$msg_name}) {
        $msg = $self -> {"MESSAGES"} -> {$lang}{$msg_name};
    }
    elsif ($self -> {"MESSAGES"} -> {"en_US"}{$msg_name}) {
        $msg = $self -> {"MESSAGES"} -> {"en_US"}{$msg_name} .
        " [Translation missing!] ";
    }
    else {
        $msg = ' [Message missing: ("' . $msg_name . '")]';
    };
    
    return $msg;
};




#-----------------------------------------------------------------------------
# Function get
#
# Purpose  : Returns translated message.
#
# Input    : $msg, String, Message "constant".
# Output   : string
# Error
# handling : English translation with " [Translation missing!] " is returned
#              if the translation doesn't exist.
#
#-----------------------------------------------------------------------------
sub get {
    my ($self) = shift;
    my ($msg)  = shift;

    return $self -> get_msg ($self -> {"LANGUAGE"}, $msg);
};




#-----------------------------------------------------------------------------
# Function language_ok
#
# Purpose  : Returns true if there are translations available for the given
#              language, false otherwise.
#
# Input    : $language, String, "en_US", "fi_FI", etc.
# Output   : String
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub language_ok {
    my ($self)     = shift;
    my ($language) = shift;

    return $self -> {"MESSAGES"} -> {$language} ? 1 : 0;
};




#-----------------------------------------------------------------------------
# Function init
#
# Purpose  : Returns a hash of hash containing all translated messages.
#
# Input    : -
# Output   : hash of hash
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub init {
    my ($self) = shift;

    my (%messages) = (
        "en_US" => {
            "supported_search_engines" =>
                "Supported search engines",
            "supported_search_engines_summary..." =>
                "Supported search engines report lists all search engines " .
                "recognized by Relax.",
            "n." =>
                "n.", # n. 42 (meaning "item number 42")
            "count" =>
                "Count",
            "total" =>
                "Total",
            "generated_by" =>
                "Generated by",
            "parser" =>
                "parser",
            "relax_report" =>
                "Relax Report",
            "relax_referrer_log_analysis_report" =>
                "Relax Referrer Log Analysis Report",
            "general_summary" =>
                "General summary",
            "general_summary_summary..." =>
                "General summary report gives some basic information about " .
                "the popularity of different types of referrals.",
            "referral_type" =>
                "Referral type",
            "referred_by_search_engines" =>
                "Referred by search engines",
            "referred_by_foreign_urls" =>
                "Referred by external URLs",
            "referred_by_local_urls" =>
                "Referred by local URLs",
            "unknown_direct_from_bookmarks_typed_urls" =>
                "Unknown referrer, direct from bookmarks, robots, " .
                "and typed URLs",
            "unique_search_engines" =>
                "Unique search engines",
            "unique_query_terms" =>
                "Unique query terms",
            "unique_keywords" =>
                "Unique keywords",
            "unique_urls" =>
                "Unique URLs",
            "keyword_count" =>
                "Keyword count",
            "excluded_targets" =>
                "excluded targets",
            "excluded_referrers" =>
                "excluded referrers",
            "corrupted_lines" =>
                "corrupted lines",

            "referring_search_engines" =>
                "Referring search engines",
            "referring_search_engines_summary..." =>
                "Referring search engines report lists the search engines " .
                "visitors are using to find the analyzed site.",
            "search_engine" =>
                "Search engine",

            "referring_search_engine_locations" =>
                "Referring search engine locations",
            "referring_search_engine_locations_summary..." =>
                "Referring search engine locations report lists " .
                "the search engines and search engine locations " .
                "visitors are using to find the analyzed site.",
            "search_engine,_location" =>
                "Search engine, location",

            "search_engine_query_terms" =>
                "Search engine query terms",
            "search_engine_query_terms_summary..." =>
                "Search engine query terms report lists the query term " .
                "visitors are using in the search engines to find the " .
                "analyzed site.",
            "query_term" =>
                "Query term",

            "search_engine_keywords" =>
                "Search engine keywords",
            "search_engine_keywords_summary..." =>
                "Search engine keywords report lists the keywords " .
                "visitors are using in the search engines to find the " .
                "analyzed site.",
            "keyword" =>
                "Keyword",

            "referring_search_engines_and_query_terms" =>
                "Referring search engines and query terms",
            "referring_search_engines_and_query_terms_summary..." =>
                "Referring search engines and query terms report lists the " .
                "search engines and query terms visitors are using to find " .
                "the analyzed site.",
            "search_engine,_query_term" =>
                "Search engine, query term",

            "referring_query_terms_and_search_engine" =>
                "Referring query terms and search engines",
            "referring_query_terms_and_search_engine_summary..." =>
                "Referring query terms and search engines report lists the " .
                "query terms and search engines that visitors used to " .
                "find the analyzed site.",
            "query_term,_search_engine" =>
                "Query term, search engine",

            "all_referring_urls_excluding_search_engines" =>
                "All referring URLs to this site, excluding " .
                "recognized search engines",
            "all_referring_urls_excluding_search_engines_summary..." =>
                "All referring URLs to this site, excluding " .
                "recognized search engines report lists internal and " .
                "external referring URLs.",
            "URL" =>
                "URL",

            "local_referring_urls_to_this_site" =>
                "Local referring URLs to this site",
            "local_referring_urls_to_this_site_summary..." =>
                "Local referring URLs to this site report lists " .
                "referring URLs that belong to the analyzed site.",

            "foreign_referring_urls_to_this_site" =>
                "External referring URLs to this site",
            "foreign_referring_urls_to_this_site_summary..." =>
                "External referring URLs to this site report lists " .
                "external URLs that brought visitors to the analyzed site.",

            "all_referring_urls_to_this_site_and_their_local_targets" =>
                "All referring URLs to this site, and their local targets",
            "all_referring_urls_to_this_site_and_their_local_targets_summary..." =>
                "All referring URLs to this site, and their local targets " .
                "report lists the internal and external referring URLs that ".
                "the visitors used to reach the local target " .
                "in the analyzed site.",
            "URL,_local_target" =>
                "URL, Local target",


            "local_referring_urls_to_this_site_and_their_local_targets" =>
                "Local referring URLs to this site, and their local targets",
            "local_referring_urls_to_this_site_and_their_local_targets_summary..." =>
                "Local referring URLs to this site, and their local targets " .
                "report list the internal referring URLs that the visitors " .
                "used to reach the local target in the analyzed site.",



            "foreign_referring_urls_to_this_site_and_their_local_targets" =>
                "External referring URLs to this site, " .
                "and their local targets",
            "foreign_referring_urls_to_this_site_and_their_local_targets_summary..." =>
                "External referring URLs to this site, and their local " .
                "targets report list the external referring URLs that the " .
                "visitors used to reach the local target in the " .
                "analyzed site.",


            "unspecified_query" => "Unspecified query, ",
            "table_of_contents" => "Table of contents",
            "table_of_contents_summary..." => "The table of contents lists " .
                "all available reports generated by Relax.",
            "available_reports" => "Available reports",

            "search_engine_query_terms_and_their_local_targets" =>
                "Search engine query terms and their local targets",
            "search_engine_query_terms_and_their_local_targets_summary..." =>
                "Search engine query terms and their local targets " .
                "report lists the query terms that visitors used in " .
                "the search engines to find the local target " .
                "(entrance page) in the analyzed site.",
            "query_term,_local_target" =>
                "Query term, local target",

            "search_engine_keywords_and_their_local_targets" =>
                "Search engine keywords and their local targets",
            "search_engine_keywords_and_their_local_targets_summary..." =>
                "Search engine keywords and their local targets " .
                "report lists the keywords that visitors used in " .
                "the search engines to find the local target " .
                "(entrance page) in the analyzed site.",

            "local_targets_and_all_referring_URLs" =>
                "Local targets and all of their referring URLs",
            "local_targets_and_all_referring_URLs_summary..." =>
                "Local targets and all of their referring URLs report " .
                "lists local targets and the internal and external URLs " .
                "that are referring into them.",
            "local_target,_referring_URL" =>
                "Local target, referring URL",


            "Next" =>
                "Next",
            "Previous" =>
                "Previous",
            "Engines_pm_file_is_rather_old" =>
                "Your Engines.pm file is rather old.\n" .
                "Please check Relax web site for updates at:\n" .
                "[RELAX_WEB_SITE]([RELAX_VERSION_NUM])",
            "A_more_recent_version_of_Relax_is_now_available..." =>
                "A more recent version of Relax is now available.\n" .
                "It includes some features not found in this version.\n" .
                "An upgrade is strongly recommended.\n" .
                "Please check Relax web site for updates at:\n" .
                "[RELAX_WEB_SITE]([RELAX_VERSION_NUM])",


            "Monday" => "Monday",
            "Tuesday" => "Tuesday",
            "Wednesday" => "Wednesday",
            "Thursday" => "Thursday",
            "Friday" => "Friday",
            "Saturday" => "Saturday",
            "Sunday" => "Sunday",

            "Mon" => "Mon",
            "Tue" => "Tue",
            "Wed" => "Wed",
            "Thu" => "Thu",
            "Fri" => "Fri",
            "Sat" => "Sat",
            "Sun" => "Sun",

            "January" => "January",
            "February" => "February",
            "March" => "March",
            "April" => "April",
            "May" => "May",
            "June" => "June",
            "July" => "July",
            "August" => "August",
            "September" => "September",
            "October" => "October",
            "November" => "November",
            "December" => "December",

            "Jan" => "Jan",
            "Feb" => "Feb",
            "Mar" => "Mar",
            "Apr" => "Apr",
            "May" => "May",
            "Jun" => "Jun",
            "Jul" => "Jul",
            "Aug" => "Aug",
            "Sep" => "Sep",
            "Oct" => "Oct",
            "Nov" => "Nov",
            "Dec" => "Dec",
        },

        "fi_FI" => {
            "supported_search_engines" =>
                "Relaxin tunnistamat hakukoneet",
            "supported_search_engines_summary..." =>
                "Relaxin tunnistamat hakukoneet -raportti listaa kaikki " .
                "ohjelman tunnistamat hakukoneet.",
            "n." =>
                "Nro",
            "count" =>
                "Mr",
            "total" =>
                "Yhteens",
            "generated_by" =>
                "Raportin teki",
            "parser" =>
                "jsennin",
            "relax_report" =>
                "Relax-raportti",
            "relax_referrer_log_analysis_report" =>
                "Relax-viittauslokiraportti",
            "general_summary" =>
                "Yleinen yhteenveto",
            "general_summary_summary..." =>
                "Yleinen yhteenveto -raportti kuvaa eri viittastyyppien " .
                "suosiota.",
            "referral_type" =>
                "Viittaustyyppi",
            "referred_by_search_engines" =>
                "Viittaukset hakukoneista",
            "referred_by_foreign_urls" =>
                "Viittaukset ulkoisista verkko-osoitteista",
            "referred_by_local_urls" =>
                "viittaukset paikallisista verkko-osoitteista",
            "unknown_direct_from_bookmarks_typed_urls" =>
                "Tuntematon viittaaja, kirjanmerkit, robotit ja " .
                "kirjoitetut osoitteet",
            "unique_search_engines" =>
                "Eri hakukoneiden lukumr",
            "unique_query_terms" =>
                "Eri hakufraasien lukumr",
            "unique_keywords" =>
                "Eri hakusanojen lukumr",
            "unique_urls" =>
                "Eri verkko-osoitteiden lukumr",
            "keyword_count" =>
                "Hakusanoja yhteens",
            "excluded_targets" =>
                "poisjtetyt kohteet",
            "excluded_referrers" =>
                "poisjtetyt viittaukset",
            "corrupted_lines" =>
                "vioittuneet",
            "referring_search_engines" =>
                "Viittaavat hakukoneet",
            "referring_search_engines_summary..." =>
                "Viittaavat hakukoneet -raportti listaa hakukoneet, joilta ".
                "vierailijat ovat lytneet analysoidulle sivustolle.",
            "search_engine" =>
                "Hakukone",

            "referring_search_engine_locations" =>
                "Viittavien hakukoneiden sijainti",
            "referring_search_engine_locations_summary..." =>
                "Viittaavien hakukoneiden sijainti -raportti listaa " .
                "hakukoneet niiden sijainnit, joilla vierailijat lysivt " .
                "analysoidulle sivustolle.",
            "search_engine,_location" =>
                "Hakukone, sijainti",

            "search_engine_query_terms" =>
                "Hakukoneissa kytetyt hakufraasit",
            "search_engine_query_terms_summary..." =>
                "Hakukoneissa kytetyt hakufraasit -raportti listaa " .
                "ne hakukoneissa kytetyt fraasit, joilla vierailijat " .
                "ovat lytneet analysoidulle sivustolle.",
            "query_term" =>
                "Hakufraasi",

            "referring_search_engines_and_query_terms" =>
                "Viittaavat hakukoneet ja niiss kytetyt hakufraasit",
            "referring_search_engines_and_query_terms_summary..." =>
                "Viittaavat hakukoneet ja niiss kytetyt hakufraasit " .
                "-raportti listaa hakukoneet ja fraasit, joilla vierailijat " .
                "lysivt analysoidulle sivustolle.",
            "search_engine,_query_term" =>
                "Hakukone, hakufraasi",

            "referring_query_terms_and_search_engine" =>
                "Hakufraasit ja hakukoneet",
            "referring_query_terms_and_search_engine_summary..." =>
                "Hakufraasit ja hakukoneet -raportti listaa hakufraasit " .
                "ja hakukoneet, joilta vierailijat ovat lytneet " .
                "analysoidulle sivustolle.",
            "query_term,_search_engine" =>
                "Hakufraasi, hakukone",

            "search_engine_keywords" =>
                "Hakukoneissa kytetyt hakusanat",
            "search_engine_keywords_summary..." =>
                "Hakukoneissa kytetyt hakusanat -raportti listaa " .
                "hakusanat, joilla vierailijat ovat lytneet " .
                "analysoidulle sivustolle.",
            "keyword" =>
                "Hakusana",

            "all_referring_urls_excluding_search_engines" =>
                "Kaikki thn palveluun viittaavat verkko-osoitteet, pl. " .
                "tunnistetut hakukoneet",
            "all_referring_urls_excluding_search_engines_summary..." =>
                "Kaikki thn palveluun viittaavat verkko-osoitteet, pl. " .
                "tunnistetut hakukoneet -raportti listaa analysoidulle " .
                "sivustolle osoittavat paikalliset ja ulkoiset " .
                "verkko-osoitteet.",
            "URL" =>
                "Verkko-osoite",

            "local_referring_urls_to_this_site" =>
                "Thn palveluun viittaavat paikalliset verkko-osoitteet",
            "local_referring_urls_to_this_site_summary..." =>
                "Thn palveluun viittaavat paikalliset verkko-osoitteet -" .
                "raportti listaa ne verkko-osoitteet, jotka " .
                "kuuluvat analysoidulle sivustolle.",

            "foreign_referring_urls_to_this_site" =>
                "Thn palveluun viittaavat ulkoiset verkko-osoitteet",
            "foreign_referring_urls_to_this_site_summary..." =>
                "Thn palveluun viittaavat ulkoiset verkko-osoitteet " .
                "-raportti listaa ne ulkoiset verkko-osoitteet, " .
                "jotka johtivat vierailijoita analysoidulle sivustolle.",


            "all_referring_urls_to_this_site_and_their_local_targets" =>
                "Kaikki thn palveluun viittaavat verkko-osoitteet ja " .
                "niiden paikalliset kohteet",
            "all_referring_urls_to_this_site_and_their_local_targets_summary..." =>
                "Kaikki thn palveluun viittaavat verkko-osoitteet ja " .
                "niiden paikalliset kohteet -raportti listaa analysoidulle " .
                "sivustolle osoittavat paikalliset ja ulkoiset " .
                "verkko-osoitteet, joita kytten vierailijat ovat " .
                "ptyneet sivuston tietyille sivuille.",
            "URL,_local_target" =>
                "Verkko-osoite, Paikallinen kohde",


            "local_referring_urls_to_this_site_and_their_local_targets" =>
                "Paikalliset thn palveluun viittaavat verkko-osoitteet ja " .
                "niiden paikalliset kohteet",
            "local_referring_urls_to_this_site_and_their_local_targets_summary..." =>
                "Paikalliset thn palveluun viittaavat verkko-osoitteet ja " .
                "niiden paikalliset kohteet -raportti listaa analysoidulle " .
                "sivustolle osoittavat paikalliset verkko-osoitteet, " .
                "joita kytten vierailijat ovat ptyneet sivuston " .
                "tietyille sivuille.",

            "foreign_referring_urls_to_this_site_and_their_local_targets" =>
                "Ulkoiset thn palveluun viittaavat verkko-osoitteet ja " .
                "niiden paikalliset kohteet",
            "foreign_referring_urls_to_this_site_and_their_local_targets_summary..." =>
                "Ulkoiset thn palveluun viittaavat verkko-osoitteet ja " .
                "niiden paikalliset kohteet -raportti listaa analysoidulle " .
                "sivustolle osoittavat ulkoiset verkko-osoitteet, " .
                "joita kytten vierailijat ovat ptyneet sivuston " .
                "tietyille sivuille.",

            "unspecified_query" =>
                "Hakusanat tuntemattomia, ",
            "table_of_contents" =>
                "Sisllysluettelo",
            "table_of_contents_summary..." => "Sisllysluetteloon on ".
                "listattu kaikki Relaxin laatimat raportit.",
            "available_reports" => "Raportit",

            "search_engine_query_terms_and_their_local_targets" =>
                "Hakukoneissa kytetyt hakufraasit ja niiden " .
                "paikalliset kohteet",
            "search_engine_query_terms_and_their_local_targets_summary..." =>
                "Hakukoneissa kytetyt hakufraasit ja niiden paikalliset " .
                "kohteet -raportti listaa ne sivuston sivut, joille " .
                "tiettyj fraaseja kyttneet vierailijat ptyivt.",
            "query_term,_local_target" =>
                "Hakufraasi, paikallinen kohde",

            "search_engine_keywords_and_their_local_targets" =>
                "Hakukoneissa kytetyt hakusanat ja niiden " .
                "paikalliset kohteet",
            "search_engine_keywords_and_their_local_targets_summary..." =>
                "Hakukoneissa kytetyt hakusanat ja niiden paikalliset " .
                "kohteet -raportti listaa ne analysoidun sivuston sivut, " .
                "joille tiettyj hakusanoja kyttneet vierailijat ptyivt.",

            "local_targets_and_all_referring_URLs" =>
                "Paikalliset kohteet ja kaikki niihin viittaavat " .
                "verkko-osoitteet",
            "local_targets_and_all_referring_URLs_summary..." =>
                "Paikalliset kohteet ja kaikki niihin viittaavat " .
                "verkko-osoitteet -raportti listaa ne verkko-osoitteet, " .
                "jotka viittaavat analysoidun sivuston paikallisiin " .
                "kohteisiin.",
            "local_target,_referring_URL" =>
                "Paikallinen kohde, viittaava verkko-osoite",

            "Next" =>
                "Seuraava",
            "Previous" =>
                "Edellinen",
            "Engines_pm_file_is_rather_old" =>
                "Engines.pm-tiedosto on jo melko vanha. " .
                "Hae pivitys Relaxin kotisivulta:\n" .
                "[RELAX_WEB_SITE]([RELAX_VERSION_NUM])",
            "A_more_recent_version_of_Relax_is_now_available..." =>
                "Uudempi versio Relaxista on julkaistu.\n" .
                "Siin on ominaisuuksia, joita ei ole tss veriossa.\n" .
                "Pivitys on suositeltava.\n" .
                "Hae pivitys Relaxin kotisivulta:\n" .
                "[RELAX_WEB_SITE]([RELAX_VERSION_NUM])",

            "Monday" => "maanantai",
            "Tuesday" => "tiistai",
            "Wednesday" => "keskiviikko",
            "Thursday" => "torstai",
            "Friday" => "perjantai",
            "Saturday" => "lauantai",
            "Sunday" => "sunnuntai",

            "Mon" => "ma",
            "Tue" => "ti",
            "Wed" => "ke",
            "Thu" => "to",
            "Fri" => "pe",
            "Sat" => "la",
            "Sun" => "su",

            "January" => "tammikuu",
            "February" => "helmikuu",
            "March" => "maaliskuu",
            "April" => "huhtikuu",
            "May" => "toukokuu",
            "June" => "keskuu",
            "July" => "heinkuu",
            "August" => "elokuu",
            "September" => "syyskuu",
            "October" => "lokakuu",
            "November" => "marraskuu",
            "December" => "joulukuu",

            "Jan" => "tammi",
            "Feb" => "helmi",
            "Mar" => "maalis",
            "Apr" => "huhti",
            "May" => "touko",
            "Jun" => "kes",
            "Jul" => "hein",
            "Aug" => "elo",
            "Sep" => "syys",
            "Oct" => "loka",
            "Nov" => "marras",
            "Dec" => "joulu",
        },
    
        # Rodolphe SIBERT <sibert[at]
        #   kyxar[dot]fr>,
        # Alain Zrcher <az[at]
        #   chanteur[dot]net>

        "fr_FR" => {
            "supported_search_engines" =>
                "Moteurs de recherche grs",
            "supported_search_engines_summary..." =>
                "Liste des moteurs de recherche connus et analyss par Relax",
            "n." =>
                "N",
            "count" =>
                "Nb",
            "total" =>
                "Total",
            "generated_by" =>
                "Gnr par",
            "parser" =>
                "Analyseur syntaxique",
            "relax_report" =>
                "Statistiques par Relax",
            "relax_referrer_log_analysis_report" =>
                "Rapport d'analyse Relax sur l'origine des visites",
            "general_summary" =>
                "Rsum gnral",
            "general_summary_summary..." =>
                "Le rsum gnral donne un apperu rapide de l'origine des connexions de votre site",
            "referral_type" =>
                "Origine",
            "referred_by_search_engines" =>
                "Rfrenc par les moteurs de recherche",
            "referred_by_foreign_urls" =>
                "Rfrenc par des URL externes",
            "referred_by_local_urls" =>
                "Rfrenc par des URL internes",
            "unknown_direct_from_bookmarks_typed_urls" =>
                "URL saisies, slectiones depuis les signets/favoris ou d'origine inconnue",
            "unique_search_engines" =>
                "Moteurs de recherche",
            "unique_query_terms" =>
                "Requtes distinctes",
            "unique_keywords" =>
                "Mots cls distincts",
            "unique_urls" =>
                "URL distinctes",
            "keyword_count" =>
                "Nombre de mots cls",
            "excluded_targets" =>
                "Cibles exclues",
            "excluded_referrers" =>
                "Origines exclues",
            "corrupted_lines" =>
                "Lignes corrompues",
            "referring_search_engines" =>
                "Moteurs de recherche",
            "referring_search_engines_summary..." =>
                "Moteurs de recherche utiliss par les internautes pour arriver sur ce site",
            "search_engine" =>
                "Moteur de recherche",
            "search_engine_query_terms" =>
                "Requtes des moteurs de recherche",
            "search_engine_query_terms_summary..." =>
                "Liste les requtes saisies par les internaute, classes par moteurs de recherche",
            "query_term" =>
                "Requte",
            "search_engine_keywords" =>
                "Mots cls des moteurs de recherche",
            "search_engine_keywords_summary..." =>
                "Liste des mots cls entrs par les internautes, classs par moteurs de recherche.",
            "keyword" =>
                "Mots cls",
            "referring_search_engines_and_query_terms" =>
                "Moteurs de recherche et requtes",
            "referring_search_engines_and_query_terms_summary..." =>
                "Liste les moteurs de recherche et requtes utiliss par les internautes pour venir sur le site",
            "search_engine,_query_term" =>
                "Moteurs de recherche, Requte",

            "referring_search_engine_locations" =>
                "Zone des moteurs de recherche",
            "referring_search_engine_locations_summary..." =>
                "Liste les zones (domaines) des moteurs de recherche utiliss par les internautes pour effectuer les requtes.",
            "search_engine,_location" =>
                "Moteur de recherche, Zone",

            "referring_search_engine_locations" =>
                "Referring search engine locations",
            "referring_search_engine_locations_summary..." =>
                "Referring search engine locations report lists " .
                "the search engines and search engine locations " .
                "visitors are using to find the analyzed site.",
            "search_engine,_location" =>
                "Search engine, location",
# Translation missing!

            "referring_query_terms_and_search_engine" =>
                "Requtes classes par moteurs de recherche",
            "referring_query_terms_and_search_engine_summary..." =>
                "Liste les requtes et moteurs de recherche utiliss par les internautes pour venir sur le site " ,
            "query_term,_search_engine" =>
                "Requte, moteur de recherche",
            "all_referring_urls_excluding_search_engines" =>
                "URL d'origine hors moteurs de recherche connus",
            "all_referring_urls_excluding_search_engines_summary..." =>
                "Liste les URL d'origine, internes comme externes, hors moteurs de recherche connus.",
            "URL" =>
                "URL",
            "local_referring_urls_to_this_site" =>
                "URL d'origine internes",
            "local_referring_urls_to_this_site_summary..." =>
				"Liste les URL d'origine internes au site (navigation interne du site)",
            "foreign_referring_urls_to_this_site" =>
                "URL d'origine externes",
            "foreign_referring_urls_to_this_site_summary..." =>
                "Liste les URL d'origine externes au site (points d'entres sur le site)",
            "all_referring_urls_to_this_site_and_their_local_targets" =>
                "URL d'origine avec leurs cibles",
            "all_referring_urls_to_this_site_and_their_local_targets_summary..." =>
				"Liste les URL d'origine, internes et externes, avec leur cible locale",
            "URL,_local_target" =>
                "URL, Cible locale",
            "local_referring_urls_to_this_site_and_their_local_targets" =>
                "URL d'origine internes avec leurs cibles",
            "local_referring_urls_to_this_site_and_their_local_targets_summary..." =>
                "Liste les URL d'origine internes avec leur cible locale (navigation interne du site)",
            "foreign_referring_urls_to_this_site_and_their_local_targets" =>
                "Liste les URL d'origine externes avec leur cible locale (point d'entre du site)",
            "foreign_referring_urls_to_this_site_and_their_local_targets_summary..." =>
				"Liste les URL d'origine externes avec leur cibles locales (points d'entre)",
            "unspecified_query" =>
                "Requte non spcifie, ",
            "table_of_contents" =>
                "Sommaire",
            "table_of_contents_summary..." => 
				 "Sommaire des analyses disponibles",
            "available_reports" => "Analyses disponibles",
            "search_engine_query_terms_and_their_local_targets" =>
                "Requtes des moteurs de recherche avec leurs cibles",
            "search_engine_query_terms_and_their_local_targets_summary..." =>
				"Liste les requtes utilises par les internautes avec les cibles locales (points d'entre)",
            "query_term,_local_target" =>
                "Requtes et cibles locales",
            "search_engine_keywords_and_their_local_targets" =>
                "Mots cls des moteurs de recherche avec leurs cibles",
            "search_engine_keywords_and_their_local_targets_summary..." =>
				"Liste les mots cls utiliss par les internautes avec les cibles locales (points d'entre)",
            "local_targets_and_all_referring_URLs" =>
                "Cibles avec leurs URL d'origine",
            "local_targets_and_all_referring_URLs_summary..." =>
				"Liste les cibles locales avec toutes leur URL d'origine, internes et externes",
            "local_target,_referring_URL" =>
                "Cibles locales, URL d'origine",
            "Next" =>
                "Suivant",
            "Previous" =>
                "Prcdent",
            "Engines_pm_file_is_rather_old" =>
                "Le fichier Engines.pm file commence  dater.\n" .
                "Merci de vrifier les mises  jour sur \n" .
                "[RELAX_WEB_SITE]([RELAX_VERSION_NUM])",
            "A_more_recent_version_of_Relax_is_now_available..." =>
                "Une version plus rcente de Relax est disponible.\n" .
                "Elle contient de nouvelles fonctionalits.\n" .
                "La mise  jour est vivement recommande.\n" .
                "Merci de vrifier les mises  jour sur \n"  .
                "[RELAX_WEB_SITE]([RELAX_VERSION_NUM])",

            # Taken from Solaris locales
            "Monday" => "lundi",
            "Tuesday" => "mardi",
            "Wednesday" => "mercredi",
            "Thursday" => "jeudi",
            "Friday" => "vendredi",
            "Saturday" => "samedi",
            "Sunday" => "dimanche",

            "Mon" => "lun",
            "Tue" => "mar",
            "Wed" => "mer",
            "Thu" => "jeu",
            "Fri" => "ven",
            "Sat" => "sam",
            "Sun" => "dim",

            "January" => "janvier",
            "February" => "fvrier",
            "March" => "mars",
            "April" => "avril",
            "May" => "mai",
            "June" => "juin",
            "July" => "juillet",
            "August" => "aot",
            "September" => "septembre",
            "October" => "octobre",
            "November" => "novembre",
            "December" => "dcembre",

            "Jan" => "jan ",
            "Feb" => "fv ",
            "Mar" => "mars",
            "Apr" => "avr ",
            "May" => "mai ",
            "Jun" => "juin",
            "Jul" => "juil",
            "Aug" => "aot",
            "Sep" => "sep ",
            "Oct" => "oct ",
            "Nov" => "nov ",
            "Dec" => "dc ",
        },
    );

    return { %messages };
};




#-----------------------------------------------------------------------------
# Function command_line_help
#
# Purpose  : Returns a string containing command line help message.
#
# Input    : -
# Output   : string
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub command_line_help {
    my ($self)    = shift;
    my ($version) = shift;

    return <<HELPMESSAGE;
$version, Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/

Usage: $0 [-a<file>|-A<file>|-c<file>|-e|-ed<string>
|f<oct>|-F<string>|-g<string>|-h|-l<rexp>|-np|-ni|-o<file>|-pi<num>
|-pw|-ra<num>|-re<num>|-rel<num1,num2>|-rgs|-rk<num>|-rkd<num1,num2>
|-rq<num>|-rqd<num1,num2>|-rsq<num1,num2>|-rsq<num1,num2>|-rra<num>
|-rrl<num>|-rrf<num>|-rrad<num>|-rrld<num>|-rrfd<num>|-rtad<num1,num2>
|-Rp<string>|-Rs<string>|-s|-sp|-sr|-S|-t|-toc|-uwb
|-wr<rexp>=<rexp> ...|-x<rexp>|-X<rexp>|-z<string>
|-C|-CS=<string>|-E|-N<string>=<string> ...|-T<file>|-W=<string>] [file ...]

 -a<file>       use <file> as archive to create incremental reports
 -A<file>       create reports from archive file <file> instead of log file
 -c<file>       read configuration info from <file>
 -e             generate JavaScript code to integrate Relax with Etsin
 -ed<string>    Use <string> as default domain name in Etsin
 -f<oct>        change file mode of all generated files to octal number <oct>
 -F<string>     format of the report
 -g<string>     choose report language
 -h             give this help
 -l<rexp>       treat URLs matching rexp as local
 -np            no percents in reports
 -ni            no item numbers in reports
 -o<file>       write result on <file> instead of standard output
 -pi<num>       refresh progress indicator after every num * 1000 lines of log
 -pw            open links in a new pop-up browser window
 -ra<num>       give all reports, discard items less than num in value, or
                display the first -num items
 -re<num>       search engine report, discard items less than num in value, or
                display the first -num items
 -rel<num1,num2> search engine location report, discard search engines and
                locations if used less than num1 and num2 times, or display
                the first -num1 search engines and -num2 locations
 -rgs           give general summary report
 -rk<num>       keyword report, discard items less than num in value, or
                display the first -num items
 -rkd<num1,num2> detailed keywords referring to local targets report, discard
                keywords and target URLs if used less than num1 and num2 times,
                or display the first -num1 targets and -num2 keywords
 -rq<num>       query term report, discard items less than num in value, or
                display the first -num items
 -rqd<num1,num2> detailed query terms referring to local targets report,
                discard query terms and targets if used less than num1 and
                num2 times, or display the first -num1 queries and -num2
                targets
 -rsq<num1,num2> referring search engines and query terms report, discard
                engines and queries if used less than num1 and num2 times,
                or display the first -num1 engines and -num2 queries
 -rqs<num1,num2> referring query terms by search engine, discard
                queries and engines if used less than num1 and num2 times,
                or display the first -num1 queries and -num2 engines
 -rtad<num1,num2> detailed local targets referred by all URLs report, discard
                targets and referring URLs if used less than num1 and num2
                times, or display the first -num1 targets and -num2 referrers
 -rra<num>      all referring URL report, discard items less than num in value,
                or display the first -num items
 -rrl<num>      local referring URL report, discard items less than num, or
                display the first -num items
 -rrf<num>      external referring URL report, discard items less than num,
                or display the first -num items
 -rrad<num1,num2> detailed all referring URL report, discard referring and
                target URLs if used less than num1 and num2 times, or
                display the first -num1 referrers and -num2 targets
 -rrld<num1,num2> detailed local referring URL report, discard referring and
                target URLs if used less than num1 and num2 times, or
                display the first -num1 referrers and -num2 targets
 -rrfd<num1,num2> detailed external referring URL report, discard referring
                and target URLs if used less than num1 and num2 times, or
                display the first -num1 referrers and -num2 targets
 -Rp<string>    use <string> as a decimal point in CVS formatted reports
 -Rs<string>    use <string> as a data separator in CVS formatted reports
 -s             standardize URL format
 -sp            strip parameters from URLs
 -sr            split reports into separate files
 -S             skip dead searh engines
 -t             generate run time information about the analysis
 -toc           add table of content
 -uwb           suggest word breaks to URLs
 -wr<rexp>=<rexp> rewrite referring URL search=replace
 -x<rexp>       exclude refs to <rexp> documents (usually \"\.(gif|jpg)\$\")
 -X<rexp>       exclude refs matching <rexp>
 -z<rexp>=<string> associate file extensions rexp to decompress applications
 -C             covert all query terms to lower case
 -CS=<string>   use character set <string> in report
 -E             prints a list of supported search engines separated by string
 -N<string>=<string> server's nickname=real name
 -T<file>       name of the template file
 -W=<string>    split phrases into words using <string> splitter
 file...        referrer log files, if none given, use standard input

Report bugs to <relax.info\@nic.fi>
HELPMESSAGE
};




#-----------------------------------------------------------------------------
# Function version
#
# Purpose  : Returns a string containing version message.
#
# Input    : -
# Output   : string
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub version {
    my ($self)    = shift;
    my ($version) = shift;

    return <<VERSIONMESSAGE;
$version
Parser: $Engines::ENGINE_PARSER_VERSION
Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
Relax comes with NO WARRANTY,
to the extent permitted by law.
You may redistribute copies of Relax
under the terms of the GNU General Public License.
For more information about these matters,
see http://www.gnu.org/copyleft/gpl.html.
VERSIONMESSAGE
};




1;

