#!/usr/bin/perl -w
#=============================================================================
#
#        Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
#
# Program name  : Relax
# Version       : 2.80 #!!!
# Purpose       : Relax analyzes logs in RefererLog or NCSA extended/combined
#                   TransferLog log format and produces reports on URLs
#                   directing to your pages, keywords used in the search
#                   engines, and search engine popularity.
#
# Platforms     : All Perl platforms should be OK. Tested: Perl v5.6.1
#                   SunOS, Perl 5.005_03, 5.8.0 Win32 (Windows 95 OSR2).
# Date          : 2003-09-08 #!!!
# Updates       : 1.00 First release. (1998-04-23)
#                 1.10 Added support for: Haku.net, Cyber 411, (1998-06-29)
#                        100hot.com, Netscape Netcenter, Sear.ch.
#                      Modified: AltaVista, Excite, Serveri, AOL Netfind,
#                        LookSmart, Webcrawler.
#                      Runs slightly faster.
#                      Added percents and totals to reports.
#                      Added exclude command line switch.
#                 1.20 Added support for: What-U-Seek, Anzwers, (1998-09-11)
#                        Encyclopdia Britannica's Internet Guide, Debriefing,
#                        HuskySearch, ProFusion, Scrub The Web, go Hip,
#                        Zoeken, MetaGer, NetGuide, Fireball, Starting Point,
#                        1Blink.com, Medialingua, Deja News, MEGAWEB, Infosel,
#                        Virgilio Ricerca, News Naver.
#                      Modified: Magellan, AOL Netfind, Ask Jeeves, HotBot.
#                      Added -E switch to prints a list of supported search
#                        engines.
#                      Added -np switch to avoid percents in reports.
#                      Added -o switch to write result on file instead of
#                        standard output.
#                      Added -e switch to generate necessary JavaScript to
#                         integrate Relax reports with unified Etsin search
#                         engine interface.
#                 2.00 Added -c switch for config file support. (1998-11-05)
#                      Added support for gzip compressed log files.
#                      Added -g switch to change language of the report.
#                      Added -toc switch to generate report's table of
#                        contents.
#                      Added -pw switch to make links rendered in a new
#                        pop-up browser window.
#                      Added -T switch to support template files.
#                      Added support for NCSA combined format log files.
#                      Added -s switch to standardize URL format.
#                      Added -N switch list web server's nicknames.
#                      Modified Document Type Declaration from HTML 3.2 to
#                        HTML 4.0 Transitional.
#                      Modified the handling of "+" characters in query
#                        strings.  Now they are treated as space.
#                      Added: Microsoft Internet Search, Kompassi, Ask Jeeves
#                        For Kids, Internet Sleuth, Direct Hit, NewHoo!,
#                        Google, Evreka.
#                      Modified: Ask Jeeves, Inference Find, Northern Light,
#                        100hot.com.
#                 2.10 Added -rk switch to generate keyword (1999-05-16)
#                        report.
#                      Added -X switch to exclude ref URLs matching <rexp>.
#                      Added -sd to strip directories from local targets.
#                      Added -rqd switch to generate detailed query terms
#                        referring to local targets report.
#                      Added -rkd switch to generate detailed keywords
#                        referring to local targets report.
#                      Added -rtad switch to generate detailed local targets
#                        referred by all URLs report.
#                      Added --help and --version switches to make Relax
#                        more compatible with GNU Coding Standards.
#                      Added -pi switch to generate a progress indicator.
#                      Added -ed switch to modify Etsin domain settings.
#                      Added -f switch to change file mode of all generated
#                        files.
#                      Added support for WebSTAR log files.
#                      Modified link generation code.  Now only http and
#                        ftp URLs are links.
#                      Partial code cleanup.
#                      Added support for automatically loaded default file,
#                        default.cfg.
#                      Improved handling of corrupted log files.
#                      Completely revamped Etsin support.
#                      Improved handling of illegal hex digits (%xx) in URLs.
#                      Added: Goo, Eule, Voila, Switchboard, Freeserve,
#                        AustroNaut, AccuFind, Apollo 7, MegaCrawler,
#                        Kimo, InfoTiger, Trade Compass, ICQ iT, Austasia.net,
#                        Index, EuroFerret, UKMax, searchUK, MetaGopher,
#                        MininCo.Com, WEB.DE, MultiCrawl.
#                      Modified: AltaVista, Google, Search.com, SavvySearch,
#                        Metacrawler, NewHoo!/Netscape Open Directory,
#                        Excite, Direct Hit, Ask Jeeves, GO Network/Infoseek
#                        Microsoft Internet Search, Anzwers, Snap!, Starting
#                        Point, Encyclopdia Britannica's Internet Guide,
#                        Sonera Plaza Haku/iNET Haku.
#                 2.20 Added -rsq switch to generate Referring search engines
#                        and query terms report.  (1999-09-09)
#                      Added -rgs switch to generate general summary report.
#                      Hex digits are now converted in referred documents
#                        also.
#                      Fixed a bug that replaced "%20" by spaces in referring
#                        URLs.
#                      Etsin support is now automatically disabled if the
#                        report is written to stdout ("-")
#                      Improved standardize_url, if possible, 
#                        bar.com/index.html and bar.com/ are consolidated.
#                      create_link modified to recognize
#                        http://2566979858/~ktmatu/ type URLs.
#                      parse_combined fixed to accept some rare log entries
#                        that were previously interpreted to be corrupted.
#                      Reports can now be configured so that only the first
#                        n entries are displayed.
#                      Partial code cleanup.
#                      Added: myGO, All The Web, NetTaxi MetaSearcher,
#                        Radar UOL, MaxiSearch, canada.com, CompuServe
#                        Internet Gateway Search, NewMalaysia,
#                        SimpleSearch.com, QuePasa.com, MCI WorldCom Internet
#                        Search, NetWORLD Connections, LinksGo, Orientation,
#                        Nomade.fr, Ecila, Francit, Hachette.net, Lokace,
#                        carrefour.net, QuiQuoiO, Yeller.COM, MyBC, AdValvas,
#                        EZResult, 7Search.com, MTV3 Internet, Yep.com.
#                      Modified: Ask Jeeves, Deja.com/Deja News,
#                        About.com/MininCo.Com, Excite, Naver.com/News Naver,
#                        GoHip!, AltaVista (AltaVista Canada), Netscape
#                        Netcenter, ICQ iT, Microsoft Internet Search,
#                        AOL Netfind, Evreka, Direct Hit, LookSmart,
#                        C4/Cyber 411, Voila.
#                 2.30 Etsin code completely rewritten (1999-10-19)
#                      Added more information to general summary report.
#                      custom_parser support added.
#                      CSS selector added to B-tag in make_two_level_report.
#                      Log files are opened in binmode to prevent middle of
#                        the file related ^Z (end of file) character troubles
#                        on Windows platforms.
#                      Added -wr switch to rewrite referring URLs
#                      Added: Dataware Knowledge Query Server, Searchalot
#                        The BigHub MegaSearch, Surfboard.nl, Thor,
#                        Searchopolis, MegaSpider.
#                      Modified: Yahoo!, ICQ iT, What-U-Seek (Intrasearch),
#                        AOL Search/AOL Netfind, Evreka, Nomade.fr, Voila,
#                        QuiQuoiO, GoTo.com, Ask Jeeves, Virgilio Ricerca.
#                 2.40 Improved suport of compressed log files. (2000-01-28)
#                        Now it is easier to analyze .bz2, .zip, .Z, etc.
#                        compressed logs.
#                      Improved standardize_url.
#                      Fixed command line ARGV decoding bug (thanks to
#                        Rodolphe Sibert <rsi@kyxar.fr>).  "-" characters
#                        in the middle of parameter are not anymore treated as
#                        start of parameter.
#                      Added French translations (Rodolphe Sibert
#                        <rsi@kyxar.fr>, Alain Zrcher <az@chanteur.net>.
#                      strip_dir deprecated.
#                      Added: RealNames, iWon, GoToWorld Search,
#                        OneCenter Spider, MyWay, supercrawler.com, TROUVEZ!,
#                        Colorstamps, PerlCrawler, Hitsgalore.com, LineOne
#                        Internet Search, Search Engines Galore, WebTop.com,
#                        SearchIt, AT&T WorldNet Search, Ixquick Metasearch,
#                        InfoSpace.com, Fujo, MatchSite, Netvigator Search
#                        Engine, ninemsn, Catcha.com.
#                      Modified: MegaSpider, Northern Light, Microsoft
#                        Internet Search, Ask Jeeves, Radar UOL,
#                        Britannica.com/Encyclopdia Britannica's Internet
#                        Guide, MetaFind.
#                 2.41 Fixed a couple of small typos that (2000-04-20)
#                        caused warnings on Perl 5.6.0.
#                      Added: BT Internet Web Search, FT.com search,
#                        Netscape Online, CEOExpress, Rambler, Yandex, Aport,
#                        GOeureka, FindWhat.
#                      Modified: Search.com, Ask Jeeves, Freeserve, ICQ iT,
#                        HotBot, MegaSpider.
#                 2.42 Fixed really strange glob bug. (2000-10-08)
#                        File::DosGlob is now used instead of standard glob.
#                      Added: Buscador.com, OpenChile, La Brjula,
#                        Allthesites.com, breathe, Cypria.com, eresMas,
#                        Forbes Finder, MonsterCrawler.com, savvy.com,
#                        Sawaal, 123India.com, IndiaInfo, METAspider,
#                        indya.com, SAPO, PlanetClick, Spray Search, in.gr,
#                        eFind.com, Jumpy, Snoopa, RocketLinks, Katalogo,
#                        Weather Underground Search, Wakano, fuzzycrawler,
#                        Surfy!, MetaMiner, InfoGrid, Everyday.com,
#                        TheKing.nl, Virgin Net Search, Web Search, Findia!,
#                        onwashington.com, rediff.com, Metor, Openfind,
#                        Dell Net, search.ch, Business.com, MetaEureka,
#                        Indiatimes.com, Sympatico-Lycos Find It, Starting
#                        Page, La Toile du Qubec, ya.com, SearchFuel,
#                        BusinessIndia.com, ePilot.com, Alberta.com, emulti,
#                        Indiaa2z.com, Quickbrowse, RedeSearch,
#                        SearchBlaster.com, SearchCactus.com, ThaiFind,
#                        TsunamiSearch, Webhelp.com, Ah-ha.com, Efamily.com,
#                        OneSearch.com, DINO-Online, Altaseek, Dpart Eclair,
#                        InfoZoid, Search66.com, IcySpicy.Com, Arianna.
#                      Modified: Ask Jeeves For Kids, Evreka, All The Web,
#                        the blue window/Sear.ch, Inference Find, Freeserve,
#                        Google, Voila, indya.com, Yahoo!, the blue window,
#                        Search.com, Anzwers, NBCi/Snap!.
#                 2.50 Added -C switch to convert all query (2001-01-10)
#                        terms to lower case.
#                      Search engine parsers are now in a separate Engines.pm
#                        module.
#                      Added Engines.pm update checks.
#                 2.51 New domain related modifications. (2001-06-12)
#                        References to lambda.nic.fi have been changed to
#                        point to ktmatu.com.
#                      A couple of reports have been renamed.
#                 2.60 Added -CS switch to select what character(2002-11-16)
#                        set is used in a HTML report file.
#                      Added -W switch to select word splitter module.
#                      Added -a switch to create incremental reports.
#                      Added -A switch to create reports from an archive file.
#                      Added -rqs switch to create Referring query terms
#                        by search engine
#                      Added -S switch to make parser to skip dead engines.
#                      custom_parser now has higher priority than the
#                        built-in parser.
#                      parse_combined recovers better from corrupted log
#                        entries.
#                      Dead search engines (marked as .invalid) are not
#                        hyperlinked in the Supported search engines
#                        report.
#                      Relax is now 40-60% faster.
#                      Memory consumption reduced.
#                 2.70 Rewrite referring URL performance improved. (2003-02-08)
#                      The Robots META tag is now set to "noindex,nofollow"
#                        in standard HTML reports.
#                      Progress indicator is more configurable.
#                      Progress indicator shows "," while generating reports.
#                      Code clean-up: Configurations are now handled by
#                        Configuration.pm module, Formatter::* modules
#                        take care of writing reports into various formats.
#                      New report layout, new report formats (XHTML, HTML,
#                        Plain text, Textless, CSV (Comma Separated Value)).
#                      Added -sr switch to split reports into separate files.
#                      Added -t switch to generate run time information.
#                      Added -uwb switch to suggest braking long URLs.
#                      Memory consumption reduced.
#                 2.80 Memory consumption reduced (caches are  (2003-09-08)
#                        flushed before the reports are generated, some data
#                        structures are only used when the reports where this
#                        data is needed is requested by the user).
#                      Faster if not all reports are generated.
#                      Added -rel switch to generate search engine location
#                        report.
#                      Code clean-up: reports are generated in Reports.pm
#                        module.
#                      Relax in now using the new Engines.pm 2.00 parser
#                        module.
#                      Strftime compatible parameters can be given to 
#                        the date construct (<!--[date:parameter]-->).
#                      Improved HTML print stylesheet.
#
# References    : Apache Module mod_log_referer RefererLog info:
#                   http://httpd.apache.org/docs/mod/mod_log_referer.html
#                 NCSA extended/combined log format (TransferLog)
#                   http://hoohoo.ncsa.uiuc.edu/docs/setup/httpd/
#                   TransferLog.html
#                   http://hoohoo.ncsa.uiuc.edu/docs/setup/httpd/
#                   LogOptions.html
#                 Configuring Apache to use NCSA extended/combined log format
#                   http://httpd.apache.org/docs/mod/mod_log_config.html
#
# Author        : Matti Tukiainen, http://ktmatu.com/mt/
#
# Acknowledgements: #!!!
#
# Availability  : New releases are available at Relax home page:
#                   http://ktmatu.com/software/relax/
#
# Mailing list  : http://groups.yahoo.com/group/relax_announce/
#                 Send a blank email message to
#                 relax_announce-subscribe@yahoogroups.com to
#                 subscribe.
#
# Installation  : See the accompanying install.html file
#
# Examples      : % ./relax.pl -ra1 /vol/www/ap-logs/referer.22-Apr
#                 Generate all reports
#                 % fgrep ktmatu /vol/www/ap-logs/referer.22-Apr |
#                   ./relax.pl -l'uta\.fi' -x'\.(gif|jpg)$' -re1 -rq5
#                   -rra10 -rrad10,10 -sp -Fhtml > report.html
#                 Generate search engine report (all entries), query term
#                   report (used at least five times), all referring URL
#                   report (referring at least ten times), referring URLs
#                   to this site used at least ten times, and their local
#                   targets, strip parameters from referring URLs.  Treat
#                   "uta.fi" as a local URL, exclude refs to gif and jpg
#                   files, and produce HTML output.
#
# Copying       : This program may be used under the terms of GNU General
#                   Public License http://www.gnu.org/copyleft/gpl.html
#
#=============================================================================

use strict;
use File::Basename; #???
use File::DosGlob 'glob';
use FindBin;
use lib "$FindBin::Bin";
use locale;
use Time::Local;
use Engines;
use Messages;
use Configuration;
use TimeDate;
use Strftime;
use Reports;
use Data::Dumper;
use Memoize;

# Cache these functions
memoize 'Engines::ext_parse_engine_data';
memoize 'standardize_url';
memoize 'rewrite_referring_url';




my ($VERSION_NUM)    = "2.80";       #!!!
my ($RELEASE_STATUS) = 1;            #!!! 0 = beta, 1 = stable
my ($RELEASE_DATE)   = "2003-09-08"; #!!!
my ($VERSION) = "Relax " . $VERSION_NUM;
my ($DEFAULT_CONFIG_FILE) = "default.cfg";




#-----------------------------------------------------------------------------
# Function standardize_url
#
# Purpose  : Standardizes URL format.  Converts URL's protocol and domain
#              portions to lower case, tries to consolidate www.foo.com
#              and foo.com, bar.com/index.html and bar.com/ addresses,
#              and replaces server's nicknames like (www1.isp.com,
#              server.isp.com, 123.1.1.1) by its real name.
#
# Input    : $referer_url, String, URL for the document that references the
#              one requested by the client.
#            $nickname_realname, Ref to hash of server's nickname, real
#               name pairs.
#            $otherref_count, Ref to hash of URL (no search engines), count
#               pairs.
# Output   : $referer_url, String, Standardized referrer
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub standardize_url {
    my ($referer_url, $nickname_realname, $otherref_count) = @_;

    my ($proto, $host, $fragment);
    my (@host_list);
    my ($tmp_host);
    my ($tmp_fragment) = "";
    my ($proto_host) = "";

    if ($referer_url =~ m#^(http|https)://([^/]+)(.*)#i) {
        ($proto, $host, $fragment) = ($1, $2, $3);
        if ($fragment eq "") {
            $host .= '/';
            $fragment = "";
        };

        $proto = lc ($proto);
        $host = lc ($host);

        if (defined ($$nickname_realname {$host})) {
            $host = $$nickname_realname {$host};
        };

        @host_list = split (/\./, $host);
        $tmp_host = $host;

        if (! (defined ($$otherref_count {$proto.'://'.$host.$fragment}))) {
            if ($host_list [0] eq "www") {
                shift (@host_list);
                $host = join (".", @host_list);

                if (not (defined (
                    $$otherref_count {$proto . '://' . $host . $fragment}))) {
                    $host = $tmp_host;
                };
            }
            else {
                $host = join (".", ("www", @host_list));
                if (not (defined ($$otherref_count {
                        $proto . '://' . $host . $fragment}))) {
                    $host = $tmp_host;
                };
            };

            $proto_host = $proto . '://' . $host;

            if ($fragment =~ /(.*\/)index\.(?:html|htm)$/) {
                $tmp_fragment = $1;

                if ($$otherref_count {$proto_host . $tmp_fragment}) {
                    $fragment = $tmp_fragment;
                }
                elsif ($$otherref_count {$proto_host . "index.html"}) {
                    $fragment = $tmp_fragment . "index.html";
                }
                elsif ($$otherref_count {$proto_host . "index.htm"}) {
                    $fragment = $tmp_fragment . "index.htm";
                };
            }
            elsif ($fragment =~ /(.*\/)$/) {
                $tmp_fragment = $1;

                if ($$otherref_count{$proto_host.$tmp_fragment."index.html"}){
                    $fragment = $tmp_fragment . "index.html";
                }
                elsif ($$otherref_count {$proto_host.$tmp_fragment."index.htm"}){
                    $fragment = $tmp_fragment . "index.htm";
                };
            };
       };

        $referer_url = $proto . '://' . $host . $fragment;
    };

    return $referer_url;
};




#-----------------------------------------------------------------------------
# Function rewrite_referring_url
#
# Purpose  : Rewrites similar but not identical referring URLs into common
#              format.
#
# Input    : $referer_url, String, Referring URL.
#            $substitution_list, Ref to List (containing Perl code to be
#              evaled).
# Output   : String
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub rewrite_referring_url {
    my ($referer_url, $substitution_list) = @_;

    my ($rexpr) = "";

    foreach $rexpr (@{$substitution_list}) {
        eval ($rexpr);
        # '$referer_url =~ ' . 's/' . $_ . '/' . ...
    };

    return $referer_url;
};




#-----------------------------------------------------------------------------
# Function detect_log_format
#
# Purpose  : Detects log file format.
#
# Input    : $sample_str, String, first line of the log file.
# Output   : Reference to a parser function.  Rerurns undef if the log format
#              is unknown.
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub detect_log_format {
    my ($sample_str) = @_;
    my ($format) = undef;

    if ($sample_str =~ m/ \-\> /) {
        $format = \&parse_referer;
    }
    elsif ($sample_str =~ m /\"([^"]*)\" \S+ \S+ \"([^"]*)\"/) {
        $format = \&parse_combined;
    }
    elsif (
        ($sample_str =~ m /(?:GET|POST|HEAD|)\t.*?\t.*?\t/) ||
        ($sample_str =~ m /\!\!LOG_FORMAT/)) {

        $format = \&parse_webstar;
    };

    return $format;
};




#-----------------------------------------------------------------------------
# Function parse_referer
#
# Purpose  : Parses a single line of log in "RefererLog" format.
#
# Input    : $log_line_str, String, a line of the log file.
# Output   : $referer_url, String, URL for the document that references the
#              one requested by the client.
#            $target_url, String, local URL to the document being referred to.
# Error
# handling : -
# Refs     : <URL:http://www.apache.org/docs/mod/mod_log_referer.html>
#
#-----------------------------------------------------------------------------
sub parse_referer {
    my ($log_line_str) = @_;

    my ($referer_url) = "";
    my ($target_url) = "";

    ($referer_url, $target_url) = split (/ \-\> /, $log_line_str);

    return ($referer_url, $target_url);
};




#-----------------------------------------------------------------------------
# Function parse_combined
#
# Purpose  : Parses a single line of log in "NCSA extended/combined" format.
#
# Input    : $log_line_str, String, a line of the log file.
# Output   : $referer_url, String, URL for the document that references the
#              one requested by the client.
#            $target_url, String, local URL to the document being referred to.
# Error
# handling : -
# Refs     : <URL:http://hoohoo.ncsa.uiuc.edu/docs/setup/httpd/
#              TransferLog.html>
#
#-----------------------------------------------------------------------------
sub parse_combined {
    my ($log_line_str) = @_;

    my ($referrer_url) = "";
    my ($target_url) = "";

    ($target_url, $referrer_url) =
        $log_line_str =~ m /\"([^"]*)\" \S+ \S+ \"([^"]*)\"/;

    if ($target_url && $referrer_url) {
        (undef, $target_url, undef) = split (/\s/, $target_url, 3);

        if ($referrer_url eq "") {
            $referrer_url = "-";
        };
    
        if (not (defined ($target_url))) {
            $target_url = "-";
        };
    };

    return ($referrer_url, $target_url);
};




#-----------------------------------------------------------------------------
# Function parse_webstar
#
# Purpose  : Parses a single line of log in "WebSTAR" format.
#
# Input    : $log_line_str, String, a line of the log file.
# Output   : $referer_url, String, URL for the document that references the
#              one requested by the client.
#            $target_url, String, local URL to the document being referred to.
# Error
# handling : -
# Notes    : This format has not been tested extensively!
#
#-----------------------------------------------------------------------------
sub parse_webstar {
    my ($log_line_str) = @_;

    my ($referer_url) = "";
    my ($target_url) = "";

    $log_line_str =~ m /(?:GET|POST|HEAD)\t(.*?)\t(.*?)\t/;
    ($target_url, $referer_url) = ($1, $2);

    if ($referer_url eq "") {
        $referer_url = "-";
    };

    return ($referer_url, $target_url);
};




#-----------------------------------------------------------------------------
# Function expand_filenames
#
# Purpose  : Returns a list of expanded filenames.  This is not needed in
#              real operating systems where it is done automatically by the
#              shell. Required by an "OS" from Redmond.
#
# Input    : $filenames, ref to list, a list of (un)expanded filenames.
# Output   : List of expanded filenames.
# Error
# handling : Dies if the log file cannot be opened.
#
#-----------------------------------------------------------------------------
sub expand_filenames {
    my ($filenames) = @_;

    my (@glob_list) = ();
    my (@log_filename_list) = ();
    my ($log_filename_str) = "";

    foreach $log_filename_str (@{$filenames}) {
        if ($log_filename_str eq "-") {
            push (@log_filename_list, "-");
        }
        else {
            @glob_list = glob ($log_filename_str);
            if (scalar (@glob_list) == 0) {
                die ("$0: Cannot open $log_filename_str");
            }
            else {
                push (@log_filename_list, @glob_list);
            };
        };
    };

    return @log_filename_list;
};




#-----------------------------------------------------------------------------
# Function progress_indicator
#
# Purpose  : Generates a progress indicator.
#
# Input    : $line_number, Integer, Current line number.
# Output   : -
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub progress_indicator {
    my ($line_number) = @_;

    print STDERR ".";
};




#-----------------------------------------------------------------------------
# Function resolve_unzip_app
#
# Purpose  : Checks if there is a decompression application associated to
#              filename.
#
# Input    : $filename_str, String, Filename.
#            $ext_app, Ref to a hash of extension application pairs.
# Output   : name of the decompression application.
# Error
# handling : returns undef if no extension matches.
#
#-----------------------------------------------------------------------------
sub resolve_unzip_app {
    my ($filename_str) = shift;
    my ($ext_app)      = shift;
    
    my ($ext) = "";
    my ($unzip_app) = undef;

    for $ext (keys (%{$ext_app})) {
        if ($filename_str =~ /$ext/) {
            $unzip_app = $ext_app -> {$ext};
            last;
        };
    };

    return $unzip_app;
};




#-----------------------------------------------------------------------------
# Function analyze_log
#
# Purpose  : Reads through the log file and analyzes it.
#
# Input    : $command_line, Ref to Hash, Parsed command line information.
#            $unspecified_msg_str, String, Localized string for
#              "Unspecified query,".
# Output   : Array of refs to hash
# Error
# handling : Dies if the log file cannot be opened.
#
#-----------------------------------------------------------------------------
sub analyze_log {
    my ($command_line, $unspecified_msg_str) = @_;

    my (%otherref_count) = ();
    my (%ref_target_count) = ();
    my (%keyword_target_count) = ();
    my (%query_target_count) = ();
    my (%general_count) = ();
    my (%sengine_keyword_count) = ();
    my (%sengine_location_count) = ();
    my ($referer_url) = "";
    my ($target_url) = "";
    my ($query_str) = "";
    my ($engine_str) = "";
    my ($location_str) = "";
    my ($exclude_target_str) = $command_line -> {"exclude_targets"};
    my ($exclude_ref_str) = $command_line -> {"exclude_refs"};
    my ($log_filename_str) = "";
    my (@log_filename_list) = expand_filenames (
        $command_line -> {"ref_log_fname_list"}
    );
    my ($pid);
    my (@key_word_list) = [];
    my ($keyword) = "";
    my ($parser);
    my (@rewrite_referring_url_substitution_list) = map {
        '$referer_url =~ ' .
        's/' . $_ . '/' . $$command_line {"rewrite_referring_url"}{$_} . '/'
    } keys (%{$$command_line {"rewrite_referring_url"}});
    my ($rexpr) = "";
    my ($unzip_application) = undef;
    my ($word_splitter) = $command_line -> {"word_splitter_obj"};
    my ($archive_filename_str) = $command_line -> {"archive_filename"} ||
        $command_line -> {"read_archive_filename"} || "";
    my ($parsed_engine_data); # ref to hash
    my ($need_query_target_count) = (
        $command_line -> {"rqs_1_report"}
        ||
        $command_line -> {"rqd_1_report"}
        ||
        $command_line -> {"query_report"}
        ||
        $command_line -> {"rqd_1_report"}
        ||
        $command_line -> {"rgs_report"}
    );
    my ($need_otherref_count) = (
        $command_line -> {"rra_report"}
        ||
        $command_line -> {"rrl_report"}
        ||
        $command_line -> {"rrf_report"}
        ||
        $command_line -> {"rrad_1_report"}
        ||
        $command_line -> {"rrld_1_report"}
        ||
        $command_line -> {"rrfd_1_report"}
        ||
        $command_line -> {"rgs_report"}
    );
    my ($need_ref_target_count) = (
        $command_line -> {"rrad_1_report"}
        ||
        $command_line -> {"rrld_1_report"}
        ||
        $command_line -> {"rrfd_1_report"}
        ||
        $command_line -> {"rtad_1_report"}
    );
    my ($need_keyword_target_count) = (
        $command_line -> {"rgs_report"}
        ||
        $command_line -> {"keyword_report"}
        ||
        $command_line -> {"rkd_1_report"}
    );
    my ($need_sengine_keyword_count) = (
        $command_line -> {"rgs_report"}
        ||
        $command_line -> {"rsq_1_report"}
        ||
        $command_line -> {"rqs_1_report"}
        ||
        $command_line -> {"engine_report"}
        ||
        $command_line -> {"rel_1_report"}
    );

    if ($archive_filename_str) {
        if (-e $archive_filename_str) {
            my ($archive) = slurp_file ($archive_filename_str);
            eval ($$archive); # slurp inside eval caused strange errors...
            warn $@ if $@;
            undef $archive; # make memory recycleable
        }
        elsif ($command_line -> {"read_archive_filename"}) {
            warn ($0 . ': Archive file "'.$archive_filename_str.'" not found');
        };
    };

    @log_filename_list = () if ($command_line -> {"read_archive_filename"});

    foreach $log_filename_str (@log_filename_list) {

        $unzip_application = resolve_unzip_app (
            $log_filename_str,
            $command_line -> {"unzip_app"}
        );

        if ($unzip_application) {
            $log_filename_str = $unzip_application . " " .
                $log_filename_str . " |";
            $pid = open (LOGFILE, $log_filename_str);
            if (not (defined ($pid))) {
                die ("$0: Cannot open $log_filename_str");
            };
        }
        else {
            open (LOGFILE, "<" . $log_filename_str) ||
                die ("$0: Cannot open $log_filename_str");
        };

        binmode (LOGFILE);

        while (<LOGFILE>) {
            chomp ($_);
            # print ($_, "\n");
            $general_count {"lines_in_log"}++;

            if ($. == 1) { # input line number
                $parser = (
                    $command_line -> {"custom_parser"} ||
                    detect_log_format ($_) ||
                    undef
                );

                if (not (defined ($parser))) {
                    warn ($0.": Unknown log format: ".$log_filename_str,"\n");
                    $general_count {"unknown_log_format_files"}++;
                    if ($log_filename_str =~ /\|$/) {
                        # dont break the pipe
                        while (<LOGFILE>) {};
                    };
                    last;
                };
            }
            elsif (
                $command_line -> {"progress_indicator"}
                &&
                ($. % $command_line -> {"progress_indicator"} == 0)
            ) {
                progress_indicator ($.);
            };
            
            ($referer_url, $target_url) = &$parser ($_);

            if (not (defined ($referer_url) && defined ($target_url))) {
                warn ("\nCorrupted log: " . $log_filename_str . " line $.\n");
                $general_count {"corrupted_lines"}++;
                next;
            };


            if ($exclude_target_str ne "") {
                if ($target_url =~ /$exclude_target_str/o) {
                    $general_count {"target_excluded"}++;
                    next;
                };
            };
            if ($exclude_ref_str ne "") {
                if ($referer_url =~ /$exclude_ref_str/o) {
                    $general_count {"referrer_excluded"}++;
                    next;
                };
            };

            if (scalar (@rewrite_referring_url_substitution_list)) {
                # memoized
                $referer_url = rewrite_referring_url (
                    $referer_url,
                    \@rewrite_referring_url_substitution_list
                );
            };

            $referer_url =~ s/^(http:\/\/[^\/]+):80\//$1\//o;
            $target_url  =~ s/%([a-fA-F0-9]{2})/chr (hex ($1))/ge;
            $referer_url =~ s/\#.*$//o; # strip anything after #
            $target_url  =~ s/\#.*$//o;

            $query_str = "";
            $engine_str = "";

            $parsed_engine_data = Engines::ext_parse_engine_data (
                $referer_url
            );
            $query_str = $parsed_engine_data -> {"keywords"};
            $engine_str = $parsed_engine_data -> {"search_engine_name"};
            $location_str = $parsed_engine_data -> {"search_engine_location"};


            if ($query_str ne "") {
                $query_str =~ s/\+/ /g;
                $query_str =~ s/^ //g;
                $query_str =~ s/%([a-fA-F0-9]{2})/chr (hex ($1))/ge;
                $query_str =~ s/[\n\r]*//g;

                if ($command_line -> {"lower_case_query_terms"}) {
                    $query_str = lc ($query_str);
                };

                if ($need_query_target_count) {
                    $query_target_count {$query_str}{$target_url}++;
                };

                if ($need_sengine_keyword_count) {
                    $sengine_keyword_count {$engine_str}{$query_str}++;
                };

                if ($command_line -> {"rel_1_report"}) {
                    $sengine_location_count {$engine_str}{$location_str}++;
                };

                if ($need_keyword_target_count) {
                    if (not ($query_str =~ /^$unspecified_msg_str/)) {
                        @key_word_list = $word_splitter -> split ($query_str);
                        #@key_word_list=split(/[\s\+\|\;\&\,\"]/, lc($query_str));
                        #print ("\n\n", $query_str, "\n");
                        foreach $keyword (@key_word_list) {
                            #print ("\t\t", $keyword, "\n");
                            #$keyword =~ s/^[\-\+\"\ \[\(\']+//g;
                            #$keyword =~ s/[\-\+\"\ \,\]\)\'\?]+$//g;

                            $keyword = $word_splitter -> strip_junk ($keyword);
                            if ($keyword ne "") {
                                #print ("\n*", $keyword, "");
                                $keyword_target_count{$keyword}{$target_url}++;
                            };
                        };
                    };
                };
                #print ("\t", $_, "\n");
                #print ("\t", $engine_str, "\n");
                #print ("\t", $query_str, "\n");
                #print ("\n");
            }
            else {
                if ($need_otherref_count || $need_ref_target_count) {
                    if ($command_line -> {"strip_params"}) {
                        # strip all parameters after "?"
                        $referer_url =~ s/\?.*$//o;
                        $target_url  =~ s/\?.*$//o;
                    };

                    $referer_url =~ s/#.+$//o;  # strip anchors
                    $referer_url =~ s/%([a-fA-F0-9]{2})/chr (hex ($1))/ge;
                    $referer_url =~ s/[ ]+$//g;
                    $referer_url =~ s/ /%20/g;
                };

                if ($need_otherref_count || $need_ref_target_count) {
                    if ($command_line -> {"standardize_urls"}) {
                        $referer_url = standardize_url (
                            $referer_url,
                            $command_line -> {"nickname_realname"},
                            \%otherref_count
                        );
                    };

                    $otherref_count {$referer_url}++;
                };

                if ($need_ref_target_count) {
                    $ref_target_count {$referer_url}{$target_url}++;
                };
            };
        };
       
        close (LOGFILE);
    };

    return (
        \%query_target_count,
        \%otherref_count,
        \%ref_target_count,
        \%keyword_target_count,
        \%sengine_keyword_count,
        \%general_count,
        \%sengine_location_count
    );
};




#-----------------------------------------------------------------------------
# Function swap_hash_of_hash_keys
#
# Purpose  : Swaps key1 and key2 of hash of hash. 
#            'http://www.uta.fi/~ktmatu/rates.html' => {
#                '/~ktmatu/rate-search.html' => '2',
#                '/~ktmatu/rate-about.html' => '8', #*
#                '/~ktmatu/rate-whatsnew.html' => '1', ...
#            becomes:
#           '/~ktmatu/rate-about.html' => {
#               'http://www.uta.fi/~ktmatu/rates.html' => '8'
#           },
#           ...
#
# Input    : $hash, ref to a hash.
# Output   : ref to a hash.
# Error
# handling :
#
#-----------------------------------------------------------------------------
sub swap_hash_of_hash_keys {
    my ($hash) = @_;

    my (%swapped_hash);
    my ($key1, $key2);

    # foreach $key1 (keys (%$hash)) { # requires more memory
    while ($key1 = each (%$hash)) {
        # print ($key1, "\n");
        # print ("\t", $hash -> {$key1}, "\n");

        # for $key2 (keys (%{$hash -> {$key1}})) {
        while ($key2 = each (%{$hash -> {$key1}})) {
            # print ("\t\t", $key2, " ", $hash -> {$key1}{$key2}, "\n");
            $swapped_hash {$key2}{$key1} = $hash -> {$key1}{$key2};
        };
    };

    return \%swapped_hash;
};




#-----------------------------------------------------------------------------
# Function summarize_hash_of_hash
#
# Purpose  : Summarizes hash of hash:
#            'foo' => {
#                '/~ktmatu/zxc.html' => '1',
#                '/~ktmatu/asd.html' => '6'
#            },
#            'bar' => {
#                '/~ktmatu/hgf.html' => '1'
#            }
#
#            becomes:
#            'foo' => '7'
#            'bar' => '1'
#
# Input    : $hash, ref to a hash.
# Output   : ref to a hash.
# Error
# handling :
#
#-----------------------------------------------------------------------------
sub summarize_hash_of_hash {
    my ($hash) = @_;

    my (%summarized_hash);
    my ($sum);
    my ($key1, $key2);

    while ($key1 = each (%$hash)) {
    # foreach $key1 (keys (%$hash)) {
        #print ($key1, "\n");
        $sum = 0;

        # for $key2 (keys (%{$hash -> {$key1}})) {
        while ($key2 = each (%{$hash -> {$key1}})) {
            #print ("\t\t", $key2, " ", $hash -> {$key1}{$key2}, "\n");
            $sum += $hash -> {$key1}{$key2};
            $summarized_hash {$key1} = $sum;
        };
        #print ("\t\t", $sum, "\n");
    };

    return \%summarized_hash;
};




#-----------------------------------------------------------------------------
# Function sum_hash
#
# Purpose  : Adds up values in a hash of key-value pairs.
#
# Input    : $hash, ref to a hash.
#            $keys, ref to a list which contains keys to items to be added.
# Output   : scalar.
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub sum_hash {
    my ($hash, $keys) = @_;
    my ($item);
    my ($sum) = 0;

    foreach $item (@$keys) {
        $sum += $hash -> {$item};
    };

    return $sum;
};




#-----------------------------------------------------------------------------
# Function slurp_file
#
# Purpose  : Reads whole file into a string.
#
# Input    : $source_filename, String, Name of the file.
# Output   : Reference to a String
# Error
# handling : Dies if the file cannot be opened.
#
#-----------------------------------------------------------------------------
sub slurp_file {
    my ($source_filename) = @_;
    my ($data, $tmp_input_record_separator);

    $tmp_input_record_separator = $/;

    undef ($/);
    open (IN, "<$source_filename") ||
        die ($0 , ": Cannot open $source_filename\n");
    $data = <IN>;
    close (IN);

    $/ = $tmp_input_record_separator;

    return \$data;
};




#-----------------------------------------------------------------------------
# Function chk_update_status
#
# Purpose  : Checks if there are newer versions of relax.pl or Engines.pm.
#
#            $messages, Object, get internationalized strings.
#            $relax_version_num, String, Relax version number ("1.23").
#            $relax_release_date, String, Relax release date ("2003-07-29").
#            $relax_release_status, Integer, 0 = beta, 1 = stable
#            $engines_parser_date, String, Engines.pm release
#              date ("2003-07-29")
#            $engines_relax_web_site, String, Relax website for updates.
#            $engines_current_beta_relax_date, String, Release date of
#              the latest Relax beta in Engines.pm.
#            $engines_current_stable_relax_date, String, Release date of
#              the latest Relax stable in Engines.pm.
# Output   : Messages are written into stderr.
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub chk_update_status {
    my (
        $messages,
        $relax_version_num,
        $relax_release_date,
        $relax_release_status,
        $engines_parser_date,
        $engines_relax_web_site,
        $engines_current_beta_relax_date,
        $engines_current_stable_relax_date
    ) = @_;
    
    my ($year, $month, $day) =
        $engines_parser_date =~ /(\d\d\d\d)\-(\d\d)\-(\d\d)/;
    my ($parser_date) = timelocal (0, 0, 0, $day, $month - 1, $year - 1900);

    my ($old_engines_msg) = $messages -> get ("Engines_pm_file_is_rather_old");
    my ($new_relax_msg) = $messages -> 
        get ("A_more_recent_version_of_Relax_is_now_available...");

    $old_engines_msg =~ s/\[RELAX_VERSION_NUM\]/$relax_version_num/;
    $old_engines_msg =~ s/\[RELAX_WEB_SITE\]/$engines_relax_web_site/;
    $new_relax_msg =~ s/\[RELAX_VERSION_NUM\]/$relax_version_num/;
    $new_relax_msg =~ s/\[RELAX_WEB_SITE\]/$engines_relax_web_site/;

    if ((time () - $parser_date) > (120 * 24 * 60 * 60)) {
        print STDERR "\n", $old_engines_msg, "\a\n\n";
    };

    if ($relax_release_status == 0) { # beta
        if (
            ($engines_current_beta_relax_date gt $relax_release_date)
            ||
            ($engines_current_stable_relax_date gt $relax_release_date)
        ) {
            print STDERR "\n", $new_relax_msg, "\a\n\n";
        };
    }
    elsif ($engines_current_stable_relax_date gt $relax_release_date) {
        print STDERR "\n", $new_relax_msg, "\a\n\n";
    };
};




#-----------------------------------------------------------------------------
# Function write_archive
#
# Purpose  : Writes the contents of all relevant variables to an archive
#              file. This makes it possible to create incremental reports.
#
# Input    : $filename_str, String, Filename of the data file.
#            $query_count, $query_target_count, ..., variables to be saved.
# Output   : Messages are written into stderr.
# Error
# handling : Dies if the file cannot be opened.
#
#-----------------------------------------------------------------------------
sub write_archive {
    my (
        $filename_str, $query_target_count, $otherref_count,
        $ref_target_count, $keyword_target_count, $sengine_keyword_count,
        $sengine_location_count, $general_count
    ) = @_;

    my ($header_str) = <<HEADER;
#=============================================================================
# Copyright (C) 1998-2003, Matti Tukiainen, http://ktmatu.com/mt/
# All rights reserved.
#
# Relax archive file.
#
# This file is generated by Relax $VERSION_NUM. Don't edit if you are not
# sure what you are doing.
#
# Relax is distributed with NO WARRANTY OF ANY KIND. No author or distributor
# accepts any responsibility for the consequences of using it, or for whether
# it serves any particular purpose or works at all, unless he or she says so
# in writing. Refer to the GNU General Public License (the "License") for
# full details.
#=============================================================================
HEADER

    open (OUTPUT, ">" . $filename_str) ||
        die ("$0: " . "Cannot open " . $filename_str . "\n");

    my ($d) = Data::Dumper -> new (
        [$query_target_count, $otherref_count, $ref_target_count,
         $keyword_target_count, $sengine_keyword_count,
         $sengine_location_count, $general_count],
        [qw(*query_target_count *otherref_count *ref_target_count
         *keyword_target_count *sengine_keyword_count
         *sengine_location_count *general_count)]
    );
    $d -> Indent (1);

    (print OUTPUT $header_str, "\n");
    (print OUTPUT $d -> Dump, "\n");

    (print OUTPUT "# This Is The End\n"); # The Doors (1967), The End
    (print OUTPUT "# Beautiful Friend\n");
    (print OUTPUT "# This Is The End\n");
    (print OUTPUT "# My Only Friend, The End\n");

    close (OUTPUT);
};




#-----------------------------------------------------------------------------
# Function init_strftime
#
# Purpose  : Initializes date and time formatter.
#
# Input    : $messages, Translated messages object.
# Output   : Initialized date and time formatter.
# Error
# handling : -
#
#-----------------------------------------------------------------------------
sub init_strftime {
    my ($messages) = @_;

    my (@abbr_wday) = (
        $messages -> get ("Mon"),
        $messages -> get ("Tue"),
        $messages -> get ("Wed"),
        $messages -> get ("Thu"),
        $messages -> get ("Fri"),
        $messages -> get ("Sat"),
        $messages -> get ("Sun")
    );
    my (@full_wday) = (
        $messages -> get ("Monday"),
        $messages -> get ("Tuesday"),
        $messages -> get ("Wednesday"),
        $messages -> get ("Thursday"),
        $messages -> get ("Friday"),
        $messages -> get ("Saturday"),
        $messages -> get ("Sunday")
    );
    my (@abbr_month) = (
        $messages -> get ("Jan"),
        $messages -> get ("Feb"),
        $messages -> get ("Mar"),
        $messages -> get ("Apr"),
        $messages -> get ("May"),
        $messages -> get ("Jun"),
        $messages -> get ("Jul"),
        $messages -> get ("Aug"),
        $messages -> get ("Sep"),
        $messages -> get ("Oct"),
        $messages -> get ("Nov"),
        $messages -> get ("Dec")
    );
    my (@full_month) = (
        $messages -> get ("January"),
        $messages -> get ("February"),
        $messages -> get ("March"),
        $messages -> get ("April"),
        $messages -> get ("May"),
        $messages -> get ("June"),
        $messages -> get ("July"),
        $messages -> get ("August"),
        $messages -> get ("September"),
        $messages -> get ("October"),
        $messages -> get ("November"),
        $messages -> get ("December")
    );

    my ($time_format_obj) = Strftime -> new (
        \@abbr_wday,
        \@full_wday,
        \@abbr_month,
        \@full_month
    );

    return $time_format_obj;
};


# Yeah, I know, this looks really horrible...
my ($config) = Configuration -> new ();
my (%command_line) = %{$config -> get_all ()}; #!!!
my ($report) = Reports -> new ();
my ($query_count);
my ($query_target_count);
my ($engine_count);
my ($otherref_count);
my ($ref_target_count);
my ($target_count);
my ($keyword_count);
my ($keyword_target_count);
my ($sengine_keyword_count);
my ($general_count);
my (%general_stats) = ();
my ($sengine_location_count);
my ($local_url_rexp);
my ($keys_ref);
my (@local_keys);
my (@foreign_keys);
my ($item);
my (@toc_lol) = ();
my ($messages) = undef;
my ($query_keys_ref);
my ($keyword_keys_ref);

my ($start_prog) = (times)[0];
my ($start_reports);
my ($start_report);

my ($formatter) = $config -> get ("formatter");

$messages = Messages -> new (
    -language => $config -> get ("language")
);

if (! $messages -> language_ok ($config -> get ("language"))) {
    die ($0.": Unknown language ".'"' . $config -> get ("language") .'"',"\n");
};

$formatter -> begin_document (
    -title    => $messages -> get ("relax_report"),
    -language => $config -> get ("language"),
    -split    => $config -> get ("split_reports")
);

my ($strftime) = init_strftime ($messages);

$Engines::UNSPECIFIED_MSG = $messages -> get ("unspecified_query");

my ($base_host_url, $base_path_url) =
    $config -> get ("etsin_domain") =~ m|(.*?//[^/]*)(.*)|;
$base_host_url = "" unless ($base_host_url);
$base_path_url = "" unless ($base_path_url);

if ($config -> get ("skip_dead_engines")) {
    $Engines::SKIP_DEAD_ENGINES = 1;
};

if ($config -> get ("help")) {
    die ($messages -> command_line_help ($VERSION));
}
elsif ($config -> get ("version")) {
    die ($messages -> version ($VERSION));
}
else {
    chk_update_status (
        $messages,
        $VERSION_NUM,
        $RELEASE_DATE,
        $RELEASE_STATUS,
        $Engines::ENGINE_PARSER_DATE,
        $Engines::RELAX_WEB_SITE,
        $Engines::CURRENT_BETA_RELAX_DATE,
        $Engines::CURRENT_STABLE_RELAX_DATE
    );

    ($query_target_count, $otherref_count, $ref_target_count,
    $keyword_target_count, $sengine_keyword_count, $general_count,
    $sengine_location_count) =
        analyze_log (\%command_line, $Engines::UNSPECIFIED_MSG);
#print `ps -l -p $$`;
    Memoize::flush_cache ('Engines::ext_parse_engine_data');
    Memoize::flush_cache ('standardize_url');
    Memoize::flush_cache ('rewrite_referring_url');

    if ($config -> get ("archive_filename")) {
        write_archive (
            $config -> get ("archive_filename"),
            $query_target_count,
            $otherref_count,
            $ref_target_count,
            $keyword_target_count,
            $sengine_keyword_count,
            $sengine_location_count,
            $general_count
        );
    };

    print STDERR "\nParser:\t", ((times)[0] - $start_prog), " sec\n"
        if ($config -> get ("run_time"));
    $start_reports = (times)[0];

    $start_report = (times)[0];
    $engine_count = summarize_hash_of_hash ($sengine_keyword_count);
    $keys_ref = $report -> rev_numeric_sort_hash_by_value ($engine_count);
    $general_stats {"uniq_sengines"} = scalar (@$keys_ref);
    $general_stats {"sengine_refs"} =
        $report -> sum_hash ($engine_count, $keys_ref);
    if ($config -> get ("engine_report") || $config -> get ("all_reports")) {
        $report -> make_one_level_report (
            $formatter,
            $messages,
            $config,
            $engine_count,
            $keys_ref,
            $config -> get ("engine_report"),
            3,
            "",
            "re",
            $messages -> get ("referring_search_engines"),
            $messages -> get ("referring_search_engines_summary..."),
            $messages -> get ("search_engine"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get ("referring_search_engines"),
                "re"
            ]
        );
    };
    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "re:\t", sprintf ('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));

    $start_report = (times)[0];
    if ($config -> get ("rel_1_report") || $config -> get ("all_reports")) {
        $report -> make_two_level_report (
            $formatter,
            $messages,
            $config,
            $engine_count,
            $sengine_location_count,
            $keys_ref,
            $config -> get ("rel_1_report"),
            $config -> get ("rel_2_report"),
            3,
            0,
            $base_host_url,
            $base_path_url,
            "rel",
            $messages -> get ("referring_search_engine_locations"),
            $messages ->
                get ("referring_search_engine_locations_summary..."),
            $messages -> get ("search_engine,_location"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get ("referring_search_engine_locations"),
                "rel"
            ]
        );
    };
    # Make memory recycleable
    undef $sengine_location_count;

    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rel:\t", sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    if ($config -> get ("rsq_1_report") || $config -> get ("all_reports")) {
        $report -> make_two_level_report (
            $formatter,
            $messages,
            $config,
            $engine_count,
            $sengine_keyword_count,
            $keys_ref,
            $config -> get ("rsq_1_report"),
            $config -> get ("rsq_2_report"),
            3,
            ($config -> get ("etsin") ? 3 : 0),
            $base_host_url,
            $base_path_url,
            "rsq",
            $messages -> get ("referring_search_engines_and_query_terms"),
            $messages ->
                get ("referring_search_engines_and_query_terms_summary..."),
            $messages -> get ("search_engine,_query_term"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get ("referring_search_engines_and_query_terms"),
                "rsq"
            ]
        );
    };
    # Make memory recycleable
    undef $engine_count;
    undef $keys_ref;

    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rsq:\t", sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    $query_count = summarize_hash_of_hash ($query_target_count);
    $query_keys_ref = $report -> rev_numeric_sort_hash_by_value ($query_count);
    $general_stats {"uniq_queries"} = scalar (@$query_keys_ref);
    if ($config -> get ("rqs_1_report") || $config -> get ("all_reports")) {
        $report -> make_two_level_report (
            $formatter,
            $messages,
            $config,
            $query_count,
            swap_hash_of_hash_keys ($sengine_keyword_count),
            $query_keys_ref,
            $config -> get ("rqs_1_report"),
            $config -> get ("rqs_2_report"),
            ($config -> get ("etsin") ? 2 : 999),
            5,
            $base_host_url,
            $base_path_url,
            "rqs",
            $messages -> get ("referring_query_terms_and_search_engine"),
            $messages ->
                get ("referring_query_terms_and_search_engine_summary..."),
            $messages -> get ("query_term,_search_engine"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get ("referring_query_terms_and_search_engine"),
                "rqs"
            ]
        );
    };
    undef $sengine_keyword_count;

    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rqs:\t", sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    if ($config -> get ("query_report") || $config -> get ("all_reports")) {
        $report -> make_one_level_report (
            $formatter,
            $messages,
            $config,
            $query_count,
            $query_keys_ref,
            $config -> get ("query_report"),
            $config -> get ("etsin") ? 2 : 1,
            $base_host_url . $base_path_url,
            "rq",
            $messages -> get ("search_engine_query_terms"),
            $messages -> get ("search_engine_query_terms_summary..."),
            $messages -> get ("query_term"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get ("search_engine_query_terms"),
                "rq"
            ]
        );
    };
    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rq:\t", sprintf ('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    $keyword_count = summarize_hash_of_hash ($keyword_target_count);
    $keyword_keys_ref =
        $report -> rev_numeric_sort_hash_by_value ($keyword_count);
    $general_stats {"uniq_keywords"} = scalar (@$keyword_keys_ref);
    $general_stats {"total_keywords"} =
        sum_hash ($keyword_count, $keyword_keys_ref);
    if ($config -> get ("keyword_report") || $config -> get ("all_reports")) {
        $report -> make_one_level_report (
            $formatter,
            $messages,
            $config,
            $keyword_count,
            $keyword_keys_ref,
            $config -> get ("keyword_report"),
            $config -> get ("etsin") ? 2 : 1,
            $base_host_url . $base_path_url,
            "rk",
            $messages -> get ("search_engine_keywords"),
            $messages -> get ("search_engine_keywords_summary..."),
            $messages -> get ("keyword"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get ("search_engine_keywords"),
                "rk"
            ]
        );
    };
    undef $keyword_keys_ref;

    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rk:\t", sprintf ('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    if ($config -> get ("rqd_1_report") || $config -> get ("all_reports")) {
        $report -> make_two_level_report (
            $formatter,
            $messages,
            $config,
            $query_count,
            $query_target_count,
            $query_keys_ref,
            $config -> get ("rqd_1_report"),
            $config -> get ("rqd_2_report"),
            ($config -> get ("etsin") ? 2 : 999),
            ($config -> get ("etsin") ? 4 : 0),
            $base_host_url,
            $base_path_url,
            "rqd",
            $messages ->
                get ("search_engine_query_terms_and_their_local_targets"),
            $messages ->
                get ("search_engine_query_terms_and_their_local_targets_summary..."),
            $messages ->get ("query_term,_local_target"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get (
                    "search_engine_query_terms_and_their_local_targets"
                ),
                "rqd"
            ]
        );
    };
    undef $query_target_count;
    undef $query_count;
    undef $query_keys_ref;

    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rk:\t", sprintf ('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    $keys_ref = $report -> rev_numeric_sort_hash_by_value ($keyword_count);
    if ($config -> get ("rkd_1_report") || $config -> get ("all_reports")) {
        $report -> make_two_level_report (
            $formatter,
            $messages,
            $config,
            $keyword_count,
            $keyword_target_count,
            $keys_ref,
            $config -> get ("rkd_1_report"),
            $config -> get ("rkd_2_report"),
            ($config -> get ("etsin") ? 2 : 999),
            ($config -> get ("etsin") ? 4 : 0),
            $base_host_url,
            $base_path_url,
            "rkd",
            $messages ->
                get ("search_engine_keywords_and_their_local_targets"),
            $messages ->
                get ("search_engine_keywords_and_their_local_targets_summary..."),
            $messages -> get ("keyword"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get (
                    "search_engine_keywords_and_their_local_targets"
                ),
                "rkd"
            ]
        );
    };
    undef $keyword_count;
    undef $keyword_target_count;

    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rkd:\t", sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    $keys_ref = $report -> rev_numeric_sort_hash_by_value ($otherref_count);
    if ($config -> get ("rra_report") || $config -> get ("all_reports")) {
        $report -> make_one_level_report (
            $formatter,
            $messages,
            $config,
            $otherref_count,
            $keys_ref,
            $config -> get ("keyword_report"),
            1,
            "",
            "rra",
            $messages -> get ("all_referring_urls_excluding_search_engines"),
            $messages ->
                get ("all_referring_urls_excluding_search_engines_summary..."),
            $messages -> get ("URL"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages ->get("all_referring_urls_excluding_search_engines"),
                "rra"
            ]
        );
    };
    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rra:\t", sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $local_url_rexp = $config -> get ("local_urls");
    foreach $item (@$keys_ref) {
        if ($item =~ /$local_url_rexp/o) {
            push (@local_keys, $item);
        }
        else {
            push (@foreign_keys, $item);
        }
    };

    $general_stats {"uniq_local_referring_urls"} = scalar (@local_keys);
    $general_stats {"count_local_referring_urls"} = sum_hash (
        $otherref_count,
        \@local_keys
    );

    $general_stats {"uniq_foreign_referring_urls"} = scalar (@foreign_keys);
    $general_stats {"count_foreign_referring_urls"} = sum_hash (
        $otherref_count,
        \@foreign_keys
    );

    $general_stats {"unknown_foreign_refs"} = 0;
    $general_stats {"unknown_local_refs"} = 0;
    if (defined ($$otherref_count {"-"})) {
        if (scalar (grep {/^-$/} @foreign_keys) > 0) {
            $general_stats {"unknown_foreign_refs"} = $$otherref_count {"-"};
        }
        else {
            $general_stats {"unknown_local_refs"} = $$otherref_count {"-"};
        };
    };


    $start_report = (times)[0];
    if ($config -> get ("rrl_report") || $config -> get ("all_reports")) {
        $report -> make_one_level_report (
            $formatter,
            $messages,
            $config,
            $otherref_count,
            \@local_keys,
            $config -> get ("rrl_report"),
            1,
            "",
            "rrl",
            $messages -> get ("local_referring_urls_to_this_site"),
            $messages -> get ("local_referring_urls_to_this_site_summary..."),
            $messages -> get ("URL"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get ("local_referring_urls_to_this_site"),
                "rrl"
            ]
        );
    };
    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rrl:\t", sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    if ($config -> get ("rrf_report") || $config -> get ("all_reports")) {
        $report -> make_one_level_report (
            $formatter,
            $messages,
            $config,
            $otherref_count,
            \@foreign_keys,
            $config -> get ("rrf_report"),
            1,
            "",
            "rrf",
            $messages -> get ("foreign_referring_urls_to_this_site"),
            $messages ->
                get ("foreign_referring_urls_to_this_site_summary..."),
            $messages -> get ("URL"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get ("foreign_referring_urls_to_this_site"),
                "rrf"
            ]
        );
    };
    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rrf:\t", sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    if ($config -> get ("rrad_1_report") || $config -> get ("all_reports")) {
        $report -> make_two_level_report (
            $formatter,
            $messages,
            $config,
            $otherref_count,
            $ref_target_count,
            $keys_ref,
            $config -> get ("rrad_1_report"),
            $config -> get ("rrad_2_report"),
            1,
            0,
            $base_host_url,
            $base_path_url,
            "rrad",
            $messages ->
                get ("all_referring_urls_to_this_site_and_their_local_targets"),
            $messages ->
                get ("all_referring_urls_to_this_site_and_their_local_targets_summary..."),
            $messages -> get ("URL,_local_target"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get (
                    "all_referring_urls_to_this_site_and_their_local_targets"
                ),
                "rrad"
            ]
        );
    };
    undef $keys_ref;

    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rrad:\t",sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    if ($config -> get ("rrld_1_report") || $config -> get ("all_reports")) {
        $report -> make_two_level_report (
            $formatter,
            $messages,
            $config,
            $otherref_count,
            $ref_target_count,
            \@local_keys,
            $config -> get ("rrld_1_report"),
            $config -> get ("rrld_2_report"),
            1,
            0,
            $base_host_url,
            $base_path_url,
            "rrld",
            $messages ->
                get ("local_referring_urls_to_this_site_and_their_local_targets"),
            $messages ->
                get ("local_referring_urls_to_this_site_and_their_local_targets_summary..."),
            $messages -> get ("URL,_local_target"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get (
                    "local_referring_urls_to_this_site_and_their_local_targets"
                ),
                "rrld"
            ]
        );
    };
    undef @local_keys;

    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rrld:\t",sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    if ($config -> get ("rrfd_1_report") || $config -> get ("all_reports")) {
        $report -> make_two_level_report (
            $formatter,
            $messages,
            $config,
            $otherref_count,
            $ref_target_count,
            \@foreign_keys,
            $config -> get ("rrfd_1_report"),
            $config -> get ("rrfd_2_report"),
            1,
            0,
            $base_host_url,
            $base_path_url,
            "rrfd",
            $messages ->
                get ("foreign_referring_urls_to_this_site_and_their_local_targets"),
            $messages ->
                get ("foreign_referring_urls_to_this_site_and_their_local_targets_summary..."),
            $messages -> get ("URL,_local_target"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get (
                    "foreign_referring_urls_to_this_site_and_their_local_targets"
                ),
                "rrfd"
            ]
        );
    };
    undef $otherref_count;
    undef @foreign_keys;

    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rrfd:\t",sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    $target_count = summarize_hash_of_hash (
        swap_hash_of_hash_keys ($ref_target_count));
    $keys_ref = $report -> rev_numeric_sort_hash_by_value ($target_count);
    if ($config -> get ("rtad_1_report") || $config -> get ("all_reports")) {
        $report -> make_two_level_report (
            $formatter,
            $messages,
            $config,
            $target_count,
            swap_hash_of_hash_keys ($ref_target_count),
            $keys_ref,
            $config -> get ("rtad_1_report"),
            $config -> get ("rtad_2_report"),
            0,
            1,
            $base_host_url,
            $base_path_url,
            "rtad",
            $messages ->
                get ("local_targets_and_all_referring_URLs"),
            $messages ->
                get ("local_targets_and_all_referring_URLs_summary..."),
            $messages ->
                get ("local_target,_referring_URL"),
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get ("local_targets_and_all_referring_URLs"),
                "rtad"
            ]
        );
    };
    undef $target_count;
    undef $ref_target_count;

    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rsq:\t", sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    if ($config -> get ("rgs_report") || $config -> get ("all_reports")) {
        $report -> general_statistics_report (
            \%general_stats,
            $general_count,
            $formatter,
            $messages,
            $config
        );

        unshift (
            @toc_lol,
            [
                $messages -> get ("general_summary"),
                "rgs"
            ]
        );
    };
    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "rgs:\t", sprintf('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    $start_report = (times)[0];
    if ($config -> get ("engine_list")) {
        $report -> engine_list_report (
            $formatter,
            $messages,
            $config,
            \%Engines::ENGINE_INFO
        );

        push (
            @toc_lol,
            [
                $messages -> get ("supported_search_engines"),
                "el"
            ]
        );
    };
    print STDERR "," if ($config -> get ("progress_indicator"));
    print STDERR "el:\t", sprintf ('%.2f',(times)[0] - $start_report), " sec\n"
        if ($config -> get ("run_time"));


    if ($config -> get ("table_of_contents") or $config->get("split_reports")){
        $report -> create_table_of_contents (
            \@toc_lol,
            $formatter,
            $messages,
            $config
        );
    };

    $report -> construct_document (
        $formatter,
        $strftime,
        $messages,
        $config,
        \@toc_lol,
        $VERSION,
        $Engines::ENGINE_PARSER_VERSION
    );
};

if ($config -> get ("progress_indicator")) {
    print STDERR "\n\a";
};

# Debugging and profiling code
print STDERR "Reports:\t", ((times)[0] - $start_reports), " sec\n"
    if ($config -> get ("run_time"));
print STDERR "Total:\t", ((times)[0] - $start_prog), " sec\n"
    if ($config -> get ("run_time"));
#print `ps -l`;
#print `ps -l -p $$`;
